/* *************************************************************************************
 *
 * PROGRAM:
 *
 * 	MateSim: Mating Simulation
 *	Generate mating tables under different assumptions as monogamy or polygamy, population frequencies and mutual-propensities
 *  It works for discrete and continuous traits
 *
 * http: http://acraaj.webs6.uvigo.es/MateSim/matesim.htm
 *
 * CURRENT VERSION: 1.0
 *
 * AUTHOR:
 *
 * 	Antonio Carvajal Rodriguez (acraaj@uvigo.es) November 2017
 *
 * 	Area de Genetica. Universidad de Vigo.
 *
 * 	infomating@biosdeveloper.com
 *
 * *************************************************************************************
 *
 * DISCLAIMER:
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version. This program is
 * distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details. You should have received a copy of the GNU
 * General Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. See the file
 * "gpl.html" under the "license" directory.
 *
 * *************************************************************************************
 *
 * DESIGN: We implement two different main scenarios to represent the process of mating: Without and with replacement
 *
 *  1) Without replacement: the encounter-mating model (Gimelfarb 1988)
 *
 *     Within this scenario we use the urn scheme (Gimelfarb 1988) for the process of pair formation:
 *      1-A) Individual encounters
 *      For a total number of desired matings M the mating table is formed one by One. The algorithm is
 *         Given m=0; n1 females and n2 males, xi(0) females of type 1 and yj(0) males of type j.
 *       i) use a roulette for choosing a female i with probability proportional to the frequency pi(m), similarly for male j at frequency qj(m)
 *      ii) get a uniform number U(0,1).
 *      iii)
 *          if U<= PIij {++m; if(m<M){--n1; --n2; pi(m) = (xi(0)-m) /(n1-m); qj(m) = (yj(0)-m) /(n2-m)} else break}
 *          loop to i);
 *
 *      1-B) Mass encounters
 *         Given m=0; n1 females and n2 males, xi(0) females of type 1 and yj(0) males of type j
 *         and let n = min(n1,n2)
 *       i) For each individual from the sex having less individuals (n), say females, use a roulette for choosing a male j with probability qj(m)
 *      ii) For each of the pairs formed in i) get a uniform number U(0,1).
 *      iii)
 *          if U<= PIij {++m; if(m<M){--n1; --n2; pi(m) = (xi(0)-m) /(n1-m); qj(m) = (yj(0)-m) /(n2-m)} else break}
 *
 *      iv) after the n pairs has been tested update n=min(n1,n2) and loop to i)
 *
 *  2) With replacement
 *
 *
 * Ver 0.5 29/06/18:
 *
 * 1) Para que escriba en un fichero la distribucin completa de fenotipos tras el escalado hay que aadir el argumento:
 *
 * -writedistrib 1
 *
 * 2) Si no queremos que haga escalado (de forma que la distribucin que nos escriba con el argumento anterior sea la real sin escalar)
 * hay que aadir el argumento -scaling 0
 *
 * 3) Finalmente si queremos que slo uno de los sexos elija hay que aadir el argumento
 *
 * -onechoosy 1
 *
 * El sexo que elige siempre es el macho (segunda columna de datos)
 * si queremos que elija la hembra simplemente ponemos las hembras en las filas de machos (en vez de tag #females #males usamos #males #females
 * o si se genera una distribucin bimodal o diferente sex ratio  en el programa ponemos las caractersticas deseadas para las hembras en los parmetros de machos y viceversa
 * si queremos que elijan ambos ponemos -onechoosy 0
 *
 * Un ejemplo de lnea de comando sera
 *
 * -continuous 1 -onechoosy 1 -numfiles 10 -N 10000 -model 1 -choice 0.5 -tolerance 0.08 -bias 0 -impatience 0.0 -age_params 5 2 -1 -matings 500 -writefiles 1  -writedistrib 1
 *
 *
 * Ver 0.6 06/09/18: corrected a little bug under continuous models with undefined model parameter (default 0) there was a program break because modeltype cannot be 0 in the continuous case.
 * Ver 0.7 17/10/18:
 *  correct the FND formula for the onechoosy case with bias b<>0
 *  If onechoosy=1 (males) the Dij computation is not with absolute value because the position in the formula of the choosy sex matters
 *  the formula should use
 *  fabs(availfemales[f]-availmales[m] -bias) if we desire that the males prefer females not with the same size but with a bias
 *  e.g. availfemales[f]=availmales[m]+bias has the best mating probbility and this is true for both positive or negative bias
 *
 *  If the model is non choosy (onechoosy=0) then we use use fabs(availfemales[f]-availmales[m]) -bias for positive bias value
 *  e.g. for positive bias the best probabilities are availfemales[f]=availmales[m]+- bias or availmales[f]=availfemales[m]+-bias
 *  for negative the formula should be fabs(availfemales[f]-availmales[m]) +bias and the best probabilities coincide with the positive bias
 *  so that we just consider positive bias
 *  If the phenotypes are scaled the range of the bias must also be scaled
 *
 * Ver 0.8 23/01/19:
 *
 * Input for the assortative models changed the tag from decrfactor to choicefactor
 *
 * NEW discrete models added (model numbers from 6-9). These models have separate parameters for competence and mate choice
 *
 * Model 6: female competition parameter a generates female sexual selection; choice parameters c generates assortative mating
 * a    a   a ..a
 * 1    c   1   1
 * 1    1   c  1
 *
 * 1    1   1   c
 *
 *
 * Model 7: male competition parameter a generates male sexual selection; choice parameters c generates assortative mating
 * a    1   1 ..1
 * a    c   1   1
 * a    1   c  1
 *
 * a    1   1   c
 *
 *
 * Model 8: female competition with compound parameters, parameter a generates female sexual selection; choice parameters c generates assortative mating
 * ac   a   a ..a
 * 1    c  1   1
 * 1    1   c  1
 *
 * 1    1   1   c
 *
 * Model 9: male competition with compound parameters, parameter a generates male sexual selection; choice parameters c generates assortative mating
 *
 * ac   1   1 ..1
 * a    c   1   1
 * a    1   c   1
 *
 * a    1   1   c
 *
 *
 * Ver 0.9 /04/19:
 *
 * New: if we desire random mating with mass encounter we set modeltype = -123321
 * Previously this was done by -50 and using a user defined model with mij=1 for every ij
 * ************************************************************************************* */

#include <iostream>
#include <iomanip>
#include <algorithm>

# include <string>
# include <vector>
# include <cmath>

#include <errno.h>

# include <fstream>
# include <sstream>

#include "mmgen.hpp"
#include "makedir.h"


using namespace std;

// typedefs
    typedef struct {int male; int female;} Pair;
    typedef vector<Pair> Matings;


// ************ OUTPUT FLOW **********
	ofstream results;



void ParseArgs(int argc, char** argv,
                bool& discrete,
                bool& scaling,
                double& pi,
                double& mu2,
                double& sigma2,
                double& age_alpha,
                double& age_beta,
                double& age_gamma,
                bool& writefiles,
                double& impatientfactor,
                bool& gauss,
                bool& onechoosy,
                double& logit_alpha0,
                double& logit_alpha1,
                double& mintrait,
                double& dmax,
                double& choice,
                double& tolerance,
                double& bias,
                int& format,
                int& model,
                int& n,
                int& nprime,
                double& mmax,
                bool& onesmplround,
                bool& drift,
                int& numfiles,
                int& nameshift,
                int& numfemtypes,
                int& nummaletypes,
                double& osr,
                bool& ufreqs,
                bool& userdef,
                int& nullrow,
                int& nullcol,
                double& posfactor,
                double& choicefactor,
                double& SL,
                double& MPF,
                bool& empiricalJRpsi,
                bool& onlyresume,
                bool& writedistrib,
                bool& verbose,
                long long int& fixseed)
{

    try{
        for(int i=1; i<argc; ++i)
        {

            if((argv[i]== string("-continuous")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    discrete=atoi(argv[++i]);
                    discrete^=1; //xor the bit because if continuous is true discrete is false and viceversa
                    //if(discrete<0 || discrete >1) throw invalid_argument( "wrong continuous argument" );
                }
                continue;
            }
            if((argv[i]== string("-scaling")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    scaling=atoi(argv[++i]);

                    //if(scaling<0 || scaling >1) throw invalid_argument( "wrong writefiles argument" );
                }
                continue;
            }
            if((argv[i]== string("-format")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    format=atoi(argv[++i]);
                    if(format<0 || format >16) throw invalid_argument( "wrong format" );
                }
                continue;
            }
            if((argv[i]== string("-bimodal")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    pi=atof(argv[++i]);
                    mu2=atof(argv[++i]);
                    sigma2=atof(argv[++i]);

                    if(pi<=0 || pi >=1) throw invalid_argument( "wrong mixture argument" );
                }
                continue;
            }

//            if((argv[i]== string("-monogamy")))
//            {
//
//                if (i!=argc-1) // it is not the last argument
//                {
//                    monogamy=atoi(argv[++i]);
//                    if(monogamy<0) throw invalid_argument( "wrong monogamy argument" );
//                }
//                continue;
//            }
            if((argv[i]== string("-age_params")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    age_alpha=atof(argv[++i]);
                    age_beta=atof(argv[++i]);
                    age_gamma=atof(argv[++i]);

                    //if(gauss<0 || gauss >1) throw invalid_argument( "wrong gaussian argument" );
                }
                continue;
            }
            if((argv[i]== string("-writefiles")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    writefiles=atoi(argv[++i]);

                    //if(writefiles<0 || writefiles >1) throw invalid_argument( "wrong writefiles argument" );
                }
                continue;
            }
            if((argv[i]== string("-writedistrib")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    writedistrib=atoi(argv[++i]);

                    //if(writedistrib<0 || writedistrib >1) throw invalid_argument( "wrong writedistrib argument" );
                    //else cout<< "\Argument -writedistrib is "<<writedistrib<<endl;
                }
                continue;
            }
            if((argv[i]== string("-impatient")) || (argv[i]== string("-impatience")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    impatientfactor=atof(argv[++i]);

                }
                continue;
            }
            if((argv[i]== string("-gaussian")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    gauss=atoi(argv[++i]);

                    //if(gauss<0 || gauss >1) throw invalid_argument( "wrong gaussian argument" );
                }
                continue;
            }
            if((argv[i]== string("-onechoosy")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    onechoosy=atoi(argv[++i]);

                    //if(onechoosy<0 || onechoosy >1) throw invalid_argument( "wrong onechoosy argument" );
                }
                continue;
            }
            if((argv[i]== string("-logistic_params")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    logit_alpha0=atof(argv[++i]);
                    logit_alpha1=atof(argv[++i]);

                    //if(gauss<0 || gauss >1) throw invalid_argument( "wrong gaussian argument" );
                }
                continue;
            }
            if((argv[i]== string("-model")))
            {
// model 0 is random mating the absolute number e.g. 1 refers to female sexual selection 2 to males 3 to possitve assortative mating 4 negative etc
// if the sign is negative this means that the scheme is without replacement so
// -1 is female sexual selection matrix with the individual without replacement scheme if model is -10 the scheme is the mass encounter without replacement
                if (i!=argc-1) // it is not the last argument
                {
                    model=atoi(argv[++i]);
                    //if(model<0) throw invalid_argument( "wrong model" );
                }
                continue;
            }
            if((argv[i]== string("-matings")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    nprime=atoi(argv[++i]);
                    if(nprime<MinSampleSize) throw invalid_argument( "wrong sample size" );
                }
                continue;
            }
//            if((argv[i]== string("-MaxRounds")))
//            {
//
//                if (i!=argc-1) // it is not the last argument
//                {
//                    Rmax=atoi(argv[++i]);
//                    if(Rmax<1) throw invalid_argument( "wrong MaxRounds parameter" );
//                }
//                continue;
//            }
//            if((argv[i]== string("-Rounds")))
//            {
//                if (i!=argc-1) // it is not the last argument
//                {
//                    R=atoi(argv[++i]);
//                    if(R<1) throw invalid_argument( "wrong Rounds parameter" );
//                }
//                continue;
//            }
            if((argv[i]== string("-onematround")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    onesmplround=atoi(argv[++i]);
                    //if(drift<0) throw invalid_argument( "wrong drift argument" );
                }
                continue;
            }

            if((argv[i]== string("-MRmax"))) // maximum number of matings per round
            {
                if (i!=argc-1) // it is not the last argument
                {
                    mmax=atof(argv[++i]);
                    if(mmax<1) throw invalid_argument( "wrong number of encounters per round" );
                }
                continue;
            }
            if((argv[i]== string("-numfiles")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    numfiles=atoi(argv[++i]);
                    if(numfiles<1) throw invalid_argument( "wrong number of files to generate" );
                }
                continue;
            }
            if((argv[i]== string("-shift")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    nameshift=atoi(argv[++i]);
                    if(nameshift<0) throw invalid_argument( "wrong shift at the arguments" );
                }
                continue;
            }
            if((argv[i]== string("-N")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    n=atoi(argv[++i]);
                    if(n<2) throw invalid_argument( "wrong number of population size" );
                }
                continue;
            }
            if((argv[i]== string("-drift")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    drift=atoi(argv[++i]);
                    //if(drift<0) throw invalid_argument( "wrong drift argument" );
                }
                continue;
            }

            if((argv[i]== string("-femtypes")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    numfemtypes=atoi(argv[++i]);
                    if(numfemtypes<1) throw invalid_argument( "wrong number of female types to generate" );
                }
                continue;
            }
            if((argv[i]== string("-maletypes")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    nummaletypes=atoi(argv[++i]);
                    if(nummaletypes<1) throw invalid_argument( "wrong number of male types to generate" );
                }
                continue;
            }
            if((argv[i]== string("-osr")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    osr=atof(argv[++i]);

                    if(osr<=qzero || osr >=qbig) throw invalid_argument( "wrong sex ratio" );
                }
                continue;
            }
            if((argv[i]== string("-uniform")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    ufreqs=atoi(argv[++i]);
                    //if(ufreqs<0) throw invalid_argument( "wrong uniform freqs argument" );
                }
                continue;
            }
            if( (argv[i]== string("-userfreqs")) || (argv[i]== string("-userphenotypes")) )
            {

                if (i!=argc-1) // it is not the last argument
                {
                    userdef=atoi(argv[++i]);
                    //if(userdef<0) throw invalid_argument( "wrong userfreqs or phenotypes argument" );
                }
                continue;
            }
            if((argv[i]== string("-incrfactor")) || (argv[i]== string("-selfactor")) )
            {

                if (i!=argc-1) // it is not the last argument
                {
                    double temp=atof(argv[++i]);
                    if(temp>0) posfactor=temp; // allow postfactor <1 for the sampling over several runs under individual-encounter model
                    if(posfactor<1) throw invalid_argument( "wrong sexual selection factor argument" );
                }
                continue;
            }
            if((argv[i]== string("-choicefactor")) || (argv[i]== string("-decrfactor")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    choicefactor=atof(argv[++i]);

                    //if(choicefactor<=0) throw invalid_argument( "wrong choice factor argument" );
                }
                continue;
            }
            if((argv[i]== string("-mintrait")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    mintrait=atof(argv[++i]);
                    if(mintrait<0) throw invalid_argument( "wrong minimum trait argument" );
                }
                continue;
            }
            if((argv[i]== string("-dmax")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    dmax=atof(argv[++i]);
                    if(dmax<=0) throw invalid_argument( "wrong Dmax argument" );
                }
                continue;
            }
            if((argv[i]== string("-choice")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    choice=atof(argv[++i]);
                    if(choice<-1 || choice >1) throw invalid_argument( "wrong choice argument" );
                }
                continue;
            }
            if((argv[i]== string("-tolerance")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    tolerance=atof(argv[++i]);
                    if(tolerance<=qzero) throw invalid_argument( "wrong tolerance argument" );
                }
                continue;
            }
            if((argv[i]== string("-bias")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    bias=atof(argv[++i]);
                    //if(bias<=0) throw invalid_argument( "wrong tolerance argument" );
                }
                continue;
            }
            if((argv[i]== string("-SL")) || (argv[i]== string("-sl")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    SL=atof(argv[++i]);
                    if(SL<=0 || SL>1) throw invalid_argument( "wrong significance level (SL) argument" );
                }
                continue;
            }
            if((argv[i]== string("-MPF")) || (argv[i]== string("-mpf")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    MPF=atof(argv[++i]);
                    if(MPF<=0 || MPF>1) throw invalid_argument( "wrong MPF argument" );
                }
                continue;
            }
            if(argv[i]== string("-fixseed"))
            {
                if (i!=argc-1)//
                {
                    fixseed=atoll(argv[++i]);
                    //if(fixseed<0) throw invalid_argument( "fixseed received negative value" );
                }
                continue;
            }

            if((argv[i]== string("-eJRpsi")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    empiricalJRpsi=atoi(argv[++i]);
                    //if(empiricalJRpsi<0) throw invalid_argument( "wrong eJRpsi argument" );
                }
                continue;
            }
            if((argv[i]== string("-onlyresume")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    onlyresume=atoi(argv[++i]);
                    //if(onlyresume<0) throw invalid_argument( "wrong onlyresume argument" );
                }
                continue;
            }
            if((argv[i]== string("-verbose")))
            {

                if (i!=argc-1) // it is not the last argument
                {
                    verbose=atoi(argv[++i]);
                    //if(verbose<0) throw invalid_argument( "wrong verbose argument" );
                }
                continue;
            }
            throw invalid_argument("\n ERROR: "+string(argv[i])+ " is not a valid command line parameter\n");
        } // close for
    } // close try
    catch(const std::invalid_argument& e ){

        throw invalid_argument(e.what());
          //texterr(1,msg,true);
    }

}


int main(int argc, char* argv[])
{
//cout<<qzero<<endl;
//cout<<BIG<double>()<<endl;
//cout<<qbig<<endl; exit(0);

//vector<double>p={0,1,3};
//computepropensity(p, Logitpref);
try{
    bool verbose=false, onlyresume=false,writedistrib=false,scaling=true;
    bool discrete = true,/*monogamy=false,*/ writefiles=false, gauss=true,onechoosy=false, uniformfreqs=true, drift=false, userdef=false;
    bool impatient=false;
    bool agestructure=false;
    bool empiricalJRpsi=false;

    double MPF=0.1; // minimum phenotype frequency by population and sex
    double SL=0.05;
    double pmix=0.5, mu2=0,sigma2=1;
    double age_alpha=0, age_gamma=0, age_beta=2;
    double logit_alpha0=0;
    double logit_alpha1=1;
    double impatientfactor=0;
    int nameshift=0;
    int n=DefaultPopSize; // population size
    int nprime=DefaultSampleSize; // mating sample size
    double mmax=-1; // this will define the maximum number of encounters per round under a mass-encounter model.
    int Rmax= 1; // max mating rounds
    bool onesmplround=true; // if false indicate if there are more than one mating round
    int format=0;
    int modeltype=0; // random by default
    int numfiles=1; // number of matings at each subpopulation
    int k1=2,k2=2,nullrow=-1,nullcol=-1;
    double osr=1; // operational sex ratio
    double iniSSval=1.0, iniIsolval=1.0, posfactor=DEFAULT_FACTOR,choicefactor=DEFAULT_FACTOR; // for negative assortative

    double choice=0, tolerance=0.08;

    double Dmax =1;
    double mintrait=0, bias = 0;

    string dir="",filename="";
    string programname="MateSim";
   // *** Output directory
    string dirname="Out_"+programname;

/// Seed for the random generator
    long long int fixseed=0;

    auto seed = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count();

	//store the initial seed for later checking
	//writeseed(returnseed());

	// Arguments from main

	ParseArgs(argc, argv,discrete,scaling,pmix,mu2,sigma2,age_alpha, age_beta, age_gamma, writefiles,impatientfactor, gauss,onechoosy,logit_alpha0,logit_alpha1,mintrait,Dmax,choice, tolerance,bias, format,modeltype,n,nprime,mmax,onesmplround,drift,numfiles,nameshift,k1,k2,osr,uniformfreqs,userdef,nullrow,nullcol,posfactor,choicefactor,SL,MPF,empiricalJRpsi,onlyresume,writedistrib,verbose,fixseed);

	if(discrete){
        if(modeltype==3 && posfactor != DEFAULT_FACTOR && choicefactor == DEFAULT_FACTOR){
                string msg = "FROM VERSION 0.8 the tag -incrfactor is not valid for discrete model 3, -choicefactor should be used instead\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.err", msg,true,true);

        }
	}


    if(fixseed)
        seed =fixseed;
/// Initialize seed in the engine
    randgen.seed(seed);
    cout << "\n\nSeed: " <<seed<< endl;

	if(age_gamma <0 || age_alpha <=0) agestructure=false;
	else agestructure =true;

    if(impatientfactor !=0) impatient = true;

    if(numfiles==1)
        writefiles=true;
	///

    if(discrete)
        if(userdef && uniformfreqs){

            string msg = "User defined frequencies is incompatible with predefined uniform frequencies\n";
            cout<<msg<<endl;
            errormsg(uniformfreqs,"MateSim.err", msg,true,true);
        }

    /// use osr for computing n1 and n2

    int n1=n/(1.0+osr);
    int n2= n-n1;

	if(n1<DefaultPopSize/2 || n2<DefaultPopSize/2) drift=true; // if populationsize is lower than 500 the effect of drift is considered


/// *********************************************************************************
/// ******************************* CONTINUOUS TRAITS *******************************
/// *********************************************************************************

/// General formula for continuous traits is f = exp(alpha0 +alpha1*Z)

    if(!discrete){ // continuous and discrete trait models are very different so they are isolated parts of the code
        if(modeltype==0){

            string msg = "Continuous trait requires a preference model (see Table S2 from the manual).\n";
            cout<<msg<<endl;
            errormsg(uniformfreqs,"MateSim.err", msg,true,true);
        }


        bool EM= modeltype < 0;// detect if the desired model is Encounter-mating type (without replacement models)
        modeltype = abs(modeltype); // detect if the model is with mass encounters
        bool massEM = (modeltype>9) && EM;

        if(massEM) modeltype/=10;

        if(modeltype==3)
            gauss=false;

        filename+="MateSim_C_";
        filename+="n"+NtoS(nprime)+"_";
        if(EM){

            if(massEM){
                dirname+="_massEM";
                filename+="massEM_";
            }

            else{
                dirname+="_iEM";
                filename+="iEM_";
            }

        }

// the maximum number of matings correspond to the case when each individual of one sex mate with every individual of the other sex
        int totpairs=n1*n2;
        int maxmatings=totpairs;


        if(EM){ // without replacement

            maxmatings= min(n1,n2);

        }
        if(nprime > maxmatings){

            string msg = "\nToo few population males or females (" +NtoS(maxmatings)+ ") for too many matings (" +NtoS(nprime)+ ")\n";
            cout<<msg<<endl;
            errormsg(maxmatings,"MateSim.err", msg,true,true);

        }

        int maxattempts = maxmatings;

        vector<double>availfemales(n1,0), availmales(n2,0);
        //availfemales.reserve(n1);
        //availmales.reserve(n2);

        /// 1.- Generate nprime matings with at most n1 x n2 attempts

        //double pij=0;
        double trait=0;

        portablemkdir(dirname);
        dir=dirname+"/";
        string ofilename="";

/// Check if the distribution is bimodal

        bool bimodal= (pmix>0 && pmix < 1 && mu2!=0 && fabs(mu2) > (sigma2+1));
        if(!bimodal && mu2!=0){

            string msg = "\nWarning: note that the value of the mean (" +NtoS(mu2)+ ") and the standard deviation (" +NtoS(sigma2)+ ") may not be enough to produce a bimodal distribution"+ "\n";
            cout<<msg<<endl;
            errormsg(sigma2,"MateSim.warning", msg,true,false);
            bimodal=true;

        }

        if(userdef && bimodal){
            string msg = "\nWarning: note that the bimodality is not applied because user-defined argument is true\n";
            cout<<msg<<endl;
            errormsg(sigma2,"MateSim.warning", msg,true,false);
            bimodal=false;
        }
    /// Generate a population so that in polygamy or even monogamy the individuals can be repeated in the mating or if a mating fail
    /// the population is the same for all the runs, not so the sampling which will be different each run
        double minfem=qbig, minmale=qbig, maxfem=qlow, maxmale=qlow;
if(userdef){ // the population is read from a file. The number of phenotypes in the file must be n, the number of females n1 and males n2 is computed based on the osr

    string path="";
    string name="MateSimUserPhenotypes.txt";

    ReadUserPhenotypes(path,name, n1, n2, availfemales,minfem,maxfem, availmales,minmale, maxmale);

}
else{ // not userdef
    if(!bimodal){
            for(int f=0;f<n1;++f ){
                availfemales[f]=Z(randgen);
                if(minfem> availfemales[f]) minfem = availfemales[f];
                if(maxfem<availfemales[f]) maxfem = availfemales[f];
            }

            for(int m=0; m<n2; ++m){
                availmales[m]=Z(randgen);
                if(minmale> availmales[m]) minmale = availmales[m];
                if(maxmale<availmales[m]) maxmale = availmales[m];
            }
    }
    else{ // bimodal

        // get the proportions of the mixture
        //females
        int n1pop2 = n1*(1.0-pmix);
        int n1pop1= n1 - n1pop2;
        //males
        int n2pop2 = n2*(1.0-pmix);
        int n2pop1= n2 - n2pop2;

    assert ( (n1pop1+n1pop2 == n1) && (n2pop1+n2pop2 == n2) );
        // Females
        // from first pop (N(0,1)
        for(int f=0;f<n1pop1;++f ){
            availfemales[f]=Z(randgen);
            if(minfem> availfemales[f]) minfem = availfemales[f];
            if(maxfem<availfemales[f]) maxfem = availfemales[f];
        }
        // from second pop N(mu2,sigma2)
        for(int f=n1pop1;f<n1pop2;++f ){
            availfemales[f]=Z(randgen)*sigma2 + mu2;
            if(minfem> availfemales[f]) minfem = availfemales[f];
            if(maxfem<availfemales[f]) maxfem = availfemales[f];
        }

        // Males
        // from first pop (N(0,1)
        for(int m=0; m<n2pop1; ++m){
            availmales[m]=Z(randgen);
            if(minmale> availmales[m]) minmale = availmales[m];
            if(maxmale<availmales[m]) maxmale = availmales[m];
        }
        // from second pop N(mu2,sigma2)
        for(int m=n2pop1; m<n2pop2; ++m){
            availmales[m]=Z(randgen)*sigma2 +mu2;
            if(minmale> availmales[m]) minmale = availmales[m];
            if(maxmale<availmales[m]) maxmale = availmales[m];
        }
    } // close else bimodal
} // close else userdef


//// Gaussian models
//if(gauss && choice<0 && bias!=0){ // introduce some divergence between individuals
//    for(auto& m: availmales){
//
//        m+= bias;
//    }
//
//    minmale+=bias;
//    maxmale+=bias;
//
//}

      /// Get the min and max trait values over the two sexes
        double minobstrait = min(minfem,minmale);
        double maxobstrait = max(maxfem,maxmale);
        double orange =maxobstrait - minobstrait;
        if(orange <= qlow ){
            string msg = "\nError: no phenotypic variation in the data\n";
            cout<<msg<<endl;
            errormsg(orange,"MateSim.err", msg,true,true);
        }

        string comments="";
       /// Store the sampled matings
        vector<double> matfem(nprime, missing_value),matmale(nprime,missing_value); // 0 with 0 1 with 1,.., nprime-1 with nprime-1

      /// Store pair correlations and other statistics by file
      vector<double> correl(numfiles,missing_value);
      vector<string> filenames(numfiles,"");
      vector<double> realizedsamplesize(numfiles,0);

      int nummat=0;


if(!agestructure){
        double meanfemage=0, meanmaleage=0;

        if(gauss || modeltype<3){ /// similarity models

            if(scaling){
        /// Change the scale to be between mintrait (default is 0) and maxtrait (default maxtrait = Dmax) ) so that maxtrait = Dmax + mintrait (Dmax = maxtrait-mintrait )
        /// the formula for changing the scale from a,b to c,d is T(w) = [(d-c)*(w-a) / (b-a) ]+c

                for(auto& f:availfemales)
                    f= (Dmax*(f-minobstrait)/(orange)) + mintrait;

                for(auto& m:availmales)
                    m= (Dmax*(m-minobstrait)/(orange)) + mintrait;

                // scale also the magnitude of the bias if any. Does not depend on the position in the x axis only on the distance
                bias= Dmax*(bias)/(orange) ;
            }
        /// Absolute value of the model type define the kind of preference function
            double fC=choice;
            switch(modeltype){

            case 1: // FND

                fC/=(tolerance*Dmax);
                fC*=fC;
                fC=-fC;

                filename+="FND_";
                filename+="C"+NtoS(choice)+"_";
                filename+="s"+NtoS(tolerance)+"_";
                filename+="Dmax"+NtoS(Dmax)+"_";
                if(bias!=0)
                    filename+="bias"+NtoS(bias)+"_";
                ofilename=dir+filename;
               break;

            case 2: // DB03

                fC*= (choice/(tolerance));
                fC*=fC;
                fC=-0.5*fC;

                filename+="BD03_";
                filename+="C"+NtoS(choice)+"_";
                filename+="s"+NtoS(tolerance)+"_";
                filename+="Dmax"+NtoS(Dmax)+"_";
                if(bias!=0)
                    filename+="bias"+NtoS(bias)+"_";
                ofilename=dir+filename;

               break;

            default:
                string msg = "Wrong model number "+NtoS(modeltype)+ "\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.err", msg,true,true);
            } // close switch


            vector<double> p={0, fC,trait,choice};

            /// Compute the whole population mating distribution
            deque<double> matdistrib(totpairs,0);

            if(choice>0){ // function for positive assortative

                if(onechoosy){
                    for(int f=0;f<n1;++f){
                        int row = f*n2;
                        for(int m=0; m<n2; ++m){
                            p[2]= fabs(availfemales[f]-availmales[m]-bias); // see comment for the version 0.7
                            p[2]*=p[2];
                            matdistrib[row + m] = computepropensity(p, Gaussbasedpref);

                        }
                     }
                }
                else{ // non choosy
                    if(bias<0) bias=fabs(bias);
                    for(int f=0;f<n1;++f){
                        int row = f*n2;

                        for(int m=0; m<n2; ++m){
                            p[2]= (fabs(availfemales[f]-availmales[m])-bias); // see comment for the version 0.7
                            p[2]*=p[2];
                            matdistrib[row + m] = computepropensity(p, Gaussbasedpref);

                        }
                     }
                }


                deque<double> sortmatdistrib = matdistrib;
                sort(sortmatdistrib.begin(), sortmatdistrib.end());
                auto indxmaxpref = max_element(matdistrib.begin(), matdistrib.end());
                double maxpref= *indxmaxpref;
                //int posmaxpref = distance(matdistrib.begin(), indxmaxpref);

                assert(matdistrib[distance(matdistrib.begin(), indxmaxpref)]== sortmatdistrib[totpairs-1]);

                for(int nf=0; nf< numfiles; ++nf){
                    vector<int> AgeFem(n1,0), AgeMale(n2,0);
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
                    deque<double> tempmatdistrib;
                    if(EM && !massEM){
                        tempmatdistrib =matdistrib;
                    }
                    vector<double> tempfems = availfemales;
                    vector<double> tempmales = availmales;
                    vector<bool>availfempos(n1,true);
                    vector<bool>availmalepos(n2,true);
                    vector<int>tempposfem, tempposmale;
                    tempposfem.reserve(n1);
                    tempposmale.reserve(n2);
                    int fem=-1, male=-1;
                    int attempts=0;

                    nummat=0;

                    double updtmaxpref = maxpref;
                    double updtmaxprefmass=updtmaxpref;

                    int updtcont=0;
                    if(onechoosy){ // only one sex chooses so this sex always mate: for a given male j the position in matdistrib is matdistrib[fem*n2+j]
                        int contmale=0;
                        do{
                            int updtn1 = availfempos.size();
                            int updtn2 = availmalepos.size();

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            //int updtn = updtn1*updtn2; // choose randomly a couple in the matdistrib matrix

                            male=contmale%updtn2; // if nprime is higher than the number of males so contmale can be higher than the number of males we reset to the first male and repeat

                            while(!availmalepos[male]){
                                ++contmale;
                                male=contmale%updtn2;
                            }

                            do{ // choose a female
                                fem = uniform(randgen)*updtn1; // choose randomly a female
                                //fem = couple/ updtn2;

                            }while(!availfempos[fem]);
//cout<<contmale<<" "<<male<<endl;

                            ++AgeFem[fem];
                            ++AgeMale[male];
                            int couple= fem*updtn2+male;

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            double pij = matdistrib[couple]/ updtmaxpref;
                            if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
    //cout<<sumpref<<"\t"<<pij<<endl;
                            if(uniform(randgen) < pij ){ // mate
                                ++contmale; // for each male until the desired matings are performed
                                matfem[nummat] = tempfems[fem];
                                matmale[nummat] = tempmales[male];
                                ++nummat;
                                if(EM){
                                    if(!massEM){ // there is no replacement
                                        availfempos.erase(availfempos.begin()+fem);
                                        availmalepos.erase(availmalepos.begin()+male);

                                        tempfems.erase(tempfems.begin() + fem);
                                        tempmales.erase(tempmales.begin() + male);

                                        meanfemage += AgeFem[fem];
                                        AgeFem.erase(AgeFem.begin() + fem);
                                        meanmaleage += AgeMale[male];
                                        AgeMale.erase(AgeMale.begin() + male);

                                        /// check if we need to update updtmaxpref
                                        if(tempmatdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                    }
                                    else{ // massEM model
                                        tempposfem.push_back(fem);
                                        tempposmale.push_back(male);
                                        if(matdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        if(attempts > maxattempts){
                                            int sampled = tempposfem.size();
                                            for(int a=0; a< sampled; ++a){
                                                int tfem= tempposfem[a];
                                                int tmale=tempposmale[a];
                                                availfempos[tfem]=false;
                                                availmalepos[tmale]=false;
                                            }
                                            updtmaxpref = updtmaxprefmass;
                                            attempts=0;
                                            tempposfem.clear();
                                            tempposmale.clear();
                                        }

                                    }
                                } // close if EM

                                if(impatient){

                                    p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                                }
                            } // close if u<pij
                            ++attempts;
                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";

                                break;
                            }

                        }while(nummat<nprime);

                    } // close if one choosy
                    else{
                        do{
                            int updtn1 = availfempos.size();
                            int updtn2 = availmalepos.size();

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            int updtn = updtn1*updtn2; // choose randomly a couple in the matdistrib matrix
                            int couple;

                            do{ // choose a couple
                                couple = uniform(randgen)*updtn;
                                fem = couple/ updtn2;
                                male = couple % updtn2;

                            }while(!availfempos[fem] || !availmalepos[male]);

                            AgeFem[fem]++;
                            AgeMale[male]++;

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            double pij = matdistrib[couple]/ updtmaxpref;
                            if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
    //cout<<sumpref<<"\t"<<pij<<endl;
                            if(uniform(randgen) < pij ){ // mate
                                matfem[nummat] = tempfems[fem];
                                matmale[nummat] = tempmales[male];
                                ++nummat;
                                if(EM){
                                    if(!massEM){ // there is no replacement
                                        availfempos.erase(availfempos.begin()+fem);
                                        availmalepos.erase(availmalepos.begin()+male);

                                        tempfems.erase(tempfems.begin() + fem);
                                        tempmales.erase(tempmales.begin() + male);

                                        meanfemage += AgeFem[fem];
                                        AgeFem.erase(AgeFem.begin() + fem);
                                        meanmaleage += AgeMale[male];
                                        AgeMale.erase(AgeMale.begin() + male);

                                        /// check if we need to update updtmaxpref
                                        if(tempmatdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                    }
                                    else{ // massEM model
                                        tempposfem.push_back(fem);
                                        tempposmale.push_back(male);
                                        if(matdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        if(attempts > maxattempts){
                                            int sampled = tempposfem.size();
                                            for(int a=0; a< sampled; ++a){
                                                int tfem= tempposfem[a];
                                                int tmale=tempposmale[a];
                                                availfempos[tfem]=false;
                                                availmalepos[tmale]=false;
                                            }
                                            updtmaxpref = updtmaxprefmass;
                                            attempts=0;
                                            tempposfem.clear();
                                            tempposmale.clear();
                                        }

                                    }
                                } // close if EM

                                if(impatient){

                                    p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                                }
                            } // close if u<pij
                            ++attempts;
                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";

                                break;
                            }

                        }while(nummat<nprime);
                    } // close else onechoosy

//cout<<"Matings performed.  Writing output..."<<endl;
                    realizedsamplesize[nf]=nummat;
                    if(bimodal)
                        ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                    else
                        ofilename+=NtoS(nf+1+nameshift)+".txt";
                    comments="Positive assortative Gaussian preference";
                    if(writefiles)
                        writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                    else
                        cout<< "\nFile "<<nf+1<<" done"<<endl;

                    pair<double,double> regression;
                    if(nummat>4)
                        correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                    else
                        cout<<"no enough matings in file "<< nf+1<<endl;
                    filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";


                    /// reset
                    ofilename=dir+filename; // reset the base file name

                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
    // reset the matings vector
                    std::fill(matfem.begin(), matfem.end(), missing_value);
                    std::fill(matmale.begin(), matmale.end(), missing_value);
                    meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                    meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
                }// close for numfiles
                meanfemage/=(nummat*numfiles);
                meanmaleage/= (nummat*numfiles);
            } // close if choice > 0
            else if(choice<0){

            /// Compute the whole population mating distribution
                for(int f=0;f<n1;++f){
                    int row = f*n2;
                    for(int m=0; m<n2; ++m){
                        p[2]= (fabs(availfemales[f]-availmales[m])-Dmax);
                        p[2]*=p[2];
                        matdistrib[row + m] = computepropensity(p, Gaussbasedpref);
                    }
                 }

                deque<double> sortmatdistrib = matdistrib;
                sort(sortmatdistrib.begin(), sortmatdistrib.end());
                auto indxmaxpref = max_element(matdistrib.begin(), matdistrib.end());
                double maxpref= *indxmaxpref;
                //int posmaxpref = distance(matdistrib.begin(), indxmaxpref);

                assert(matdistrib[distance(matdistrib.begin(), indxmaxpref)]== sortmatdistrib[totpairs-1]);

                for(int nf=0; nf< numfiles; ++nf){
                    vector<int> AgeFem(n1,0), AgeMale(n2,0);
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
                    deque<double> tempmatdistrib;
                    if(EM && !massEM){
                        tempmatdistrib =matdistrib;
                    }
                    vector<double> tempfems = availfemales;
                    vector<double> tempmales = availmales;
                    vector<bool>availfempos(n1,true);
                    vector<bool>availmalepos(n2,true);
                    vector<int>tempposfem, tempposmale;
                    tempposfem.reserve(n1);
                    tempposmale.reserve(n2);
                    int fem=-1, male=-1;
                    int attempts=0;
                    nummat=0;
                    double updtmaxpref = maxpref;
                    double updtmaxprefmass=updtmaxpref;

                    int updtcont=0;
                        if(onechoosy){ // only one sex chooses so this sex always mate: for a given male j the position in matdistrib is matdistrib[fem*n2+j]
                            int contmale=0;

                            do{
                                int updtn1 = availfempos.size();
                                int updtn2 = availmalepos.size();
                                //int updtn = updtn1*updtn2; // choose randomly a couple in the matdistrib matri
                                /// Stochastic acceptance algorithm Lipowski and Lipowska 2012

                                male=contmale%updtn2; // if nprime is higher than the number of males so contmale can be higher than the number of males we reset to the first male and repeat

                                while(!availmalepos[male]){
                                    ++contmale;
                                    male=contmale%updtn2;
                                }

                                do{ // choose a couple
                                    fem = uniform(randgen)*updtn1; // choose randomly a female
                                    //fem = couple/ updtn2;

                                }while(!availfempos[fem]);


                                AgeFem[fem]++;
                                AgeMale[male]++;
                                int couple= fem*updtn2+male;
                                /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                                double pij = matdistrib[couple]/ updtmaxpref;
                                if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
        //cout<<sumpref<<"\t"<<pij<<endl;
                                if(uniform(randgen) < pij ){ // mate
                                    ++contmale; // for each male until the desired matings are performed
        //cout<<fem<<": "<<tempfems[fem]<<" Age "<<AgeFem[fem]<<"\t"<<male<<": "<<tempmales[male]<<" Age "<<AgeMale[male]<<endl;

                                    matfem[nummat] = tempfems[fem];
                                    matmale[nummat] = tempmales[male];
                                    ++nummat;
                                    if(EM){
                                        if(!massEM){ // there is no replacement
                                            availfempos.erase(availfempos.begin()+fem);
                                            availmalepos.erase(availmalepos.begin()+male);
                                            tempfems.erase(tempfems.begin() + fem);
                                            tempmales.erase(tempmales.begin() + male);
                                            meanfemage += AgeFem[fem];
                                            AgeFem.erase(AgeFem.begin() + fem);
                                            meanmaleage += AgeMale[male];
                                            AgeMale.erase(AgeMale.begin() + male);

                                            /// check if we need to update updtmaxpref
                                            if(tempmatdistrib[couple] == updtmaxpref){
                                                ++updtcont;
                                                updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                            }

                                            tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                        }
                                        else{ // massEM model
                                            tempposfem.push_back(fem);
                                            tempposmale.push_back(male);
                                            if(matdistrib[couple] == updtmaxpref){
                                                ++updtcont;
                                                updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                            }

                                            if(attempts > maxattempts){
                                                int sampled = tempposfem.size();
                                                for(int a=0; a< sampled; ++a){
                                                    int tfem= tempposfem[a];
                                                    int tmale=tempposmale[a];
                                                    availfempos[tfem]=false;
                                                    availmalepos[tmale]=false;
                                                }
                                                updtmaxpref = updtmaxprefmass;
                                                attempts=0;
                                                tempposfem.clear();
                                                tempposmale.clear();
                                            }

                                        }
                                    } // close if EM

                                    if(impatient){

                                        p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                                    }
                                } // close if u<pij

                                ++attempts;
                                if(attempts > totpairs){
                                    comments="\nIncomplete sample\n";

                                    break;
                                }


                            }while(nummat<nprime);

                    }
                    else{ // both sexes are choosy
                        do{
                            int updtn1 = availfempos.size();
                            int updtn2 = availmalepos.size();

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            int updtn = updtn1*updtn2;
                            int couple;
                            do{ // choose a female
                                couple = uniform(randgen)*updtn;
                                fem = couple/ updtn2;
                                male = couple % updtn2;

                            }while(!availfempos[fem] || !availmalepos[male]);

                            AgeFem[fem]++;
                            AgeMale[male]++;

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            double pij = matdistrib[couple]/ updtmaxpref;
                            if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
    //cout<<sumpref<<"\t"<<pij<<endl;
                            if(uniform(randgen) < pij ){ // mate

    //cout<<fem<<": "<<tempfems[fem]<<" Age "<<AgeFem[fem]<<"\t"<<male<<": "<<tempmales[male]<<" Age "<<AgeMale[male]<<endl;

                                matfem[nummat] = tempfems[fem];
                                matmale[nummat] = tempmales[male];
                                ++nummat;
                                if(EM){
                                    if(!massEM){ // there is no replacement
                                        availfempos.erase(availfempos.begin()+fem);
                                        availmalepos.erase(availmalepos.begin()+male);
                                        tempfems.erase(tempfems.begin() + fem);
                                        tempmales.erase(tempmales.begin() + male);
                                        meanfemage += AgeFem[fem];
                                        AgeFem.erase(AgeFem.begin() + fem);
                                        meanmaleage += AgeMale[male];
                                        AgeMale.erase(AgeMale.begin() + male);

                                        /// check if we need to update updtmaxpref
                                        if(tempmatdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                    }
                                    else{ // massEM model
                                        tempposfem.push_back(fem);
                                        tempposmale.push_back(male);
                                        if(matdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        if(attempts > maxattempts){
                                            int sampled = tempposfem.size();
                                            for(int a=0; a< sampled; ++a){
                                                int tfem= tempposfem[a];
                                                int tmale=tempposmale[a];
                                                availfempos[tfem]=false;
                                                availmalepos[tmale]=false;
                                            }
                                            updtmaxpref = updtmaxprefmass;
                                            attempts=0;
                                            tempposfem.clear();
                                            tempposmale.clear();
                                        }

                                    }
                                } // close if EM

                                if(impatient){

                                    p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                                }
                            } // close if u<pij

                            ++attempts;
                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";

                                break;
                            }

                        }while(nummat<nprime);
                    } // close else onechoosy

                    realizedsamplesize[nf]=nummat;

                    if(bimodal)
                        ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                    else
                        ofilename+=NtoS(nf+1+nameshift)+".txt";
                    comments="Negative assortative Gaussian preference";
                    if(writefiles)
                        writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                    else
                        cout<< "\nFile "<<nf+1<<" done"<<endl;
                    pair<double,double> regression;
                    if(nummat>4)
                        correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                    else
                        cout<<"no enough matings in file "<< nf+1<<": "<<nummat<<" pairs" <<endl;

                    filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";

                    /// reset
                    ofilename=dir+filename; // reset the base file name

                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
    // reset the matings vector
                    std::fill(matfem.begin(), matfem.end(), missing_value);
                    std::fill(matmale.begin(), matmale.end(), missing_value);
                    meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                    meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
                }// close for numfiles
                meanfemage/=(nummat*numfiles);
                meanmaleage/= (nummat*numfiles);
            } // close if choice < 0
            else{ // no preference

            /// Mating is random but still the encounter model could be different

                for(int nf=0; nf< numfiles; ++nf){
                    vector<int> AgeFem(n1,0), AgeMale(n2,0);
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed

                    vector<bool>availfempos(n1,true);
                    vector<bool>availmalepos(n2,true);
                    vector<int>tempposfem, tempposmale;
                    tempposfem.reserve(n1);
                    tempposmale.reserve(n2);
                    int fem=-1, male=-1;
                    int attempts=0;
                    nummat=0;
                    do{
                        int updtn1 = availfempos.size();
                        int updtn2 = availmalepos.size();
                        do{ // choose a female
                            fem = uniform(randgen)*updtn1;
                        }while(!availfempos[fem]);

                        do{ // choose a male
                            male = uniform(randgen)*updtn2;
                        }while(!availmalepos[male]);

                        AgeFem[fem]++;
                        AgeMale[male]++;
                         // mate
                        matfem[nummat] =availfemales[fem];
                        matmale[nummat] = availmales[male];
                        ++nummat;
                        if(EM){
                            if(!massEM){ // there is no replacement
                                availfempos[fem]=false;
                                availmalepos[male]=false;
                            }
                            else{ // massEM model
                                tempposfem.push_back(fem);
                                tempposmale.push_back(male);

                                if(attempts > maxattempts){
                                    int sampled = tempposfem.size();
                                    for(int a=0; a< sampled; ++a){
                                        int tfem= tempposfem[a];
                                        int tmale=tempposmale[a];
                                        availfempos[tfem]=false;
                                        availmalepos[tmale]=false;
                                    }
                                    attempts=0;
                                    tempposfem.clear();
                                    tempposmale.clear();
                                }

                            }
                        } // close if EM


                        ++attempts;


                    }while(nummat<nprime);

                    realizedsamplesize[nf]=nummat;
                    if(bimodal)
                        ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                    else
                        ofilename+=NtoS(nf+1+nameshift)+".txt";
                    comments="Random mating Gaussian preference";
                    if(writefiles)
                        writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                    else
                        cout<< "\nFile "<<nf+1<<" done"<<endl;
                    pair<double,double> regression;
                    if(nummat>4)
                        correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                    else
                        cout<<"no enough matings in file "<< nf+1<<endl;

                    filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";

                    /// reset
                    ofilename=dir+filename; // reset the base file name
                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
                    meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                    meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
                }// close for numfiles
                meanfemage/=(nummat*numfiles);
                meanmaleage/= (nummat*numfiles);
            } // close choice = 0 i.e. random mating

        } // close gauss
        else{ /// Logistic model: logit pij = computepropensity(p, Logitpref);
            modeltype=3;

            filename+="Log_";
            filename+="a"+NtoS(logit_alpha0)+"_b"+NtoS(logit_alpha1)+"_";
            if(impatient) filename+="Imp_";
            ofilename=dir+filename;

            vector<double> p={logit_alpha0, logit_alpha1,trait,logit_alpha0,logit_alpha1, trait};

            for(int nf=0; nf< numfiles; ++nf){
                vector<int> AgeFem(n1,0), AgeMale(n2,0);
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
                vector<double> tempfems = availfemales;
                vector<double> tempmales = availmales;

                vector<bool>availfempos(n1,true);
                vector<bool>availmalepos(n2,true);
                vector<int>tempposfem, tempposmale;
                tempposfem.reserve(n1);
                tempposmale.reserve(n2);
                int fem=-1, male=-1;
                int attempts=0;
                nummat=0;
                do{

                    int updtn1 = availfempos.size();
                    int updtn2 = availmalepos.size();

                    do{ // choose a female
                        fem = uniform(randgen)*updtn1;
                    }while(!availfempos[fem]);

                    do{ // choose a male
                        male = uniform(randgen)*updtn2;
                    }while(!availmalepos[male]);

                    AgeFem[fem]++;
                    AgeMale[male]++;
                    p[2] = tempmales[male]; // corresponds to the will of the female for the male trait
                    p[5] = tempfems[fem]; // will of the male for the female trait

                    /// pij
                    double pij = computepropensity(p, Logitpref);

                    if(uniform(randgen) < pij ){ // mate
                        matfem[nummat] =tempfems[fem];
                        matmale[nummat] = tempmales[male];
                        ++nummat;
                        if(EM){
                            if(!massEM){ // there is no replacement
//                                availfempos[fem]=false;
//                                availmalepos[male]=false;
                                availfempos.erase(availfempos.begin() + fem);
                                availmalepos.erase(availmalepos.begin() + male);
                                tempfems.erase(tempfems.begin() + fem);
                                tempmales.erase(tempmales.begin() + male);

                                meanfemage += AgeFem[fem];
                                AgeFem.erase(AgeFem.begin() + fem);
                                meanmaleage += AgeMale[male];
                                AgeMale.erase(AgeMale.begin() + male);

                            }
                            else{ // massEM model
                                tempposfem.push_back(fem);
                                tempposmale.push_back(male);

                                if(attempts > maxattempts){
                                    int sampled = tempposfem.size();
                                    for(int a=0; a< sampled; ++a){
                                        int tfem= tempposfem[a];
                                        int tmale=tempposmale[a];
                                        availfempos[tfem]=false;
                                        availmalepos[tmale]=false;
                                    }
                                    attempts=0;
                                    tempposfem.clear();
                                    tempposmale.clear();
                                }

                            }
                        } // close if EM
                        if(impatient){ // it is ok here because if nummat does not increase the impatience does not change
                            p[0]= logit_alpha0+ impatientfactor*nummat;
                            p[3]= logit_alpha0+ impatientfactor*nummat;
                        }
                    } // close if u<pij

                    ++attempts;
                    if(attempts > n1*n2){
                        comments="\nIncomplete sample\n";

                        break;
                    }

                }while(nummat<nprime);

                realizedsamplesize[nf]=nummat;
                if(bimodal)
                    ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                else
                    ofilename+=NtoS(nf+1+nameshift)+".txt";
                comments="logit preference";
                if(impatient) comments+=("_ImpatientFactor: "+NtoS(impatientfactor));
                if(writefiles)
                    writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                else
                    cout<< "\nFile "<<nf+1<<" done"<<endl;
                pair<double,double> regression;
                if(nummat>4)
                    correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                else
                    cout<<"no enough matings in file "<< nf+1<<endl;

                filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";

                /// reset
                ofilename=dir+filename; // reset the base file name
                if(massEM){
                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
                }
 // the mat vector must be refilled to the original value because if not some values not filed in this run may be still filled from the previous one
                std::fill(matfem.begin(), matfem.end(), missing_value);
                std::fill(matmale.begin(), matmale.end(), missing_value);
                meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
            }// close for numfiles

            meanfemage/=(nummat*numfiles);
            meanmaleage/= (nummat*numfiles);

        } // close logit

        double meansamplesize= accumulate(realizedsamplesize.begin(), realizedsamplesize.end(),0.0 );
        meansamplesize/=numfiles;
        if(bimodal)
            ofilename+="bCorrel.txt";
        else
            ofilename+="Correl.txt";
        writecorrelations(ofilename,programname,comments,EM, massEM,n1, n2,meansamplesize, filenames, correl,meanfemage,meanmaleage);
} // close if not age structure
else{ /// with Age Structure
        filename+="Age_a"+NtoS(age_alpha)+"_b"+NtoS(age_beta)+"_g"+NtoS(age_gamma)+"_";
        double meanfemage=0, meanmaleage=0, meansterilefemage=0, meansterilemaleage=0; // logaritmic scale

        if(gauss|| modeltype<3){ /// similarity models

            if(scaling){
            /// Change the scale to be between mintrait and maxtrait so that Dmax = maxtrait-mintrait
            /// T(w) = [(d-c)*(w-a) / (b-a) ]+c

                for(auto& f:availfemales)
                    f= (Dmax*(f-minobstrait)/(orange)) + mintrait;

                for(auto& m:availmales)
                    m= (Dmax*(m-minobstrait)/(orange)) + mintrait;

                // scale also the magnitude of the bias if any. Does not depend on the position in the x axis only on the distance
                bias= Dmax*(bias)/(orange) ;
            }
        /// Absolute value of the model type define the kind of preference function

            double fC=choice;
            switch(modeltype){

            case 1: // FND

                fC/=(tolerance*Dmax);
                fC*=fC;
                fC= -fC;

                filename+="FND_";
                filename+="C"+NtoS(choice)+"_";
                filename+="s"+NtoS(tolerance)+"_";
                filename+="Dmax"+NtoS(Dmax)+"_";
                if(bias!=0)
                    filename+="bias"+NtoS(bias)+"_";
                ofilename=dir+filename;
               break;

            case 2: // DB03: is C^4

                fC*= (choice/(tolerance));
                fC*=fC;
                fC= -0.5*fC;

                filename+="BD03_";
                filename+="C"+NtoS(choice)+"_";
                filename+="s"+NtoS(tolerance)+"_";
                filename+="Dmax"+NtoS(Dmax)+"_";
                if(bias!=0)
                    filename+="bias"+NtoS(bias)+"_";
                ofilename=dir+filename;

               break;

            default:
                string msg = "Wrong model number "+NtoS(modeltype)+ "\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.err", msg,true,true);
            } // close switch


            vector<double> p={0, fC,trait,choice};

            /// Compute the whole population mating distribution
            deque<double> matdistrib(totpairs,0);

            if(choice>0){ // function for positive assortative

                if(onechoosy){
                    for(int f=0;f<n1;++f){
                        int row = f*n2;
                        for(int m=0; m<n2; ++m){
                            p[2]= fabs(availfemales[f]-availmales[m]-bias); // see comment for the version 0.7
                            p[2]*=p[2];
                            matdistrib[row + m] = computepropensity(p, Gaussbasedpref);

                        }
                     }
                }
                else{ // nonchoosy
                    if(bias<0) bias=fabs(bias);
                    for(int f=0;f<n1;++f){
                        int row = f*n2;
                        for(int m=0; m<n2; ++m){
                            p[2]= (fabs(availfemales[f]-availmales[m])-bias); // see comment for the version 0.7
                            p[2]*=p[2];
                            matdistrib[row + m] = computepropensity(p, Gaussbasedpref);

                        }
                     }
                }

                deque<double> sortmatdistrib = matdistrib;
                sort(sortmatdistrib.begin(), sortmatdistrib.end());
                auto indxmaxpref = max_element(matdistrib.begin(), matdistrib.end());
                double maxpref= *indxmaxpref;
                //int posmaxpref = distance(matdistrib.begin(), indxmaxpref);

                assert(matdistrib[distance(matdistrib.begin(), indxmaxpref)]== sortmatdistrib[totpairs-1]);

                for(int nf=0; nf< numfiles; ++nf){

                    vector<int> AgeFem(n1,0), AgeMale(n2,0);
                    // this are necessary because the aging process

/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
                    deque<double> tempmatdistrib;
                    if(EM && !massEM){
                        tempmatdistrib =matdistrib;
                    }
                    vector<double> tempfems = availfemales;
                    vector<double> tempmales = availmales;
                    vector<bool>availfempos(n1,true);
                    vector<bool>availmalepos(n2,true);

                    vector<int>tempposfem, tempposmale;
                    tempposfem.reserve(n1);
                    tempposmale.reserve(n2);
                    int fem=-1, male=-1;
                    int attempts=0;
                    nummat=0;
                    comments="";
                    double updtmaxpref = maxpref;
                    double updtmaxprefmass=updtmaxpref;

                    int updtcont=0;
                    bool dobreak=false;
                    int countmalesterility =0,countfemsterility=0;
                    if(onechoosy){ // only one sex chooses so this sex always mate: for a given male j the position in matdistrib is matdistrib[fem*n2+j]
                        int contmale=0;
                        do{
                            bool malesterile=false;
                            bool femsterile=false;

                            int updtn1 = availfempos.size();
                            int updtn2 = availmalepos.size();
                            //int updtn = updtn1*updtn2; // choose randomly a couple in the matdistrib matrix
                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012

                            int tries=0;

                            male=contmale%updtn2; // if nprime is higher than the number of males so contmale can be higher than the number of males we reset to the first male and repeat

                            while(!availmalepos[male]){
                                ++contmale;
                                if(tries> totpairs){

                                    comments+="\nUnavailable males\n";
    //cout<<comments<<endl;
                                    dobreak=true;
                                    break;
                                }
                                male=contmale%updtn2;
                            }
                            if(dobreak) break;
                            do{ // choose a female
                                fem = uniform(randgen)*updtn1; // choose randomly a female
                                //fem = couple/ updtn2;
                                ++tries;
                                if(tries> totpairs){
                                    if(!availfempos[fem])
                                        comments+="\nUnavailable females\n";
    //cout<<comments<<endl;
                                    dobreak=true;
                                    break;
                                }

                            }while(!availfempos[fem]);

                            if(dobreak) break;


                    /// Weibull aging process: we use log(age) units
                            ++AgeFem[fem];
                            ++AgeMale[male];
                            int couple= fem*updtn2+male;
                            if(log(AgeFem[fem])>age_gamma){

                                double Rfem= exp(-pow((log(AgeFem[fem]) -age_gamma)/age_alpha, age_beta));

                                if(uniform(randgen)>= Rfem){ // has reached sterility
                                    femsterile=true;
                                    ++countfemsterility;
                                    meansterilefemage+= AgeFem[fem];

                                    if(!EM || massEM){ // because if EM and !massEM the row of this female is deleted and it is not necesarry to put to false
                                        availfempos[fem]=false;
                                    }

                                }

                            }

        //cout<<"3\t"<<updtn1<<"\t"<<updtn2<<endl;
                            if(log(AgeMale[male])>age_gamma){

                                double Rmale= exp(-pow((log(AgeMale[male]) -age_gamma)/age_alpha, age_beta));

                                if(uniform(randgen)>= Rmale){ // has reached sterility
                                    malesterile=true;
                                    ++countmalesterility;
                                    meansterilemaleage+= AgeMale[male];
                                    availmalepos[male]=false;

                                }
                            }

                            /// Deal with specifities of the EM model
                            if(EM && !massEM){


                                if(femsterile && malesterile){ // the corresponding to fem*updtn2 + male must be deleted only once

                                    /// female
                                    availfempos.erase(availfempos.begin() + fem);
                                    tempfems.erase(tempfems.begin() + fem);

                                    AgeFem.erase(AgeFem.begin() + fem);

                                   /// check if we need to update updtmaxpref
                                    int r = fem*updtn2 ;
                                    for(int m=0; m < updtn2; ++m){
                                        int cm =r+m;
                                        if(tempmatdistrib[cm] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                    }
                                    // delete the row of matinsg involving the sterile female
                                    tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );
                                    if(availfempos.empty()){
                                        comments+="\nIncomplete sample caused by female aging\n";

                                        break;
                                    }

                                    /// male

                                    /// check that not all remaining males are false
                                    bool mcheck=true;
                                    for(const auto mp: availmalepos){
                                        mcheck*= !mp;
                                    }
                                    if(mcheck){
                                        comments+="\nIncomplete sample caused by male aging\n";
                                        break;
                                    }
                                    //AgeMale.erase(AgeMale.begin() + male);
                                   /// check if we need to update updtmaxpref. ***Note here we use availfempos.size() instead updtn1 because we already erased some fems
                                    int availfemposSize=availfempos.size();
                                    for(int f=0; f < availfemposSize; ++f){
                                        if(f == fem) continue; /// ignore this case because it was already deleted
                                        int c = f*updtn2 + male;
                                        if(tempmatdistrib[c] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                    }

                                    if(availmalepos.empty()){
                                        comments+="\nIncomplete sample caused by male aging\n";

                                        break;
                                    }

                                }
                                else if(femsterile){

                                    availfempos.erase(availfempos.begin() + fem);
                                    tempfems.erase(tempfems.begin() + fem);

                                    AgeFem.erase(AgeFem.begin() + fem);

                                   /// check if we need to update updtmaxpref
                                    int r = fem*updtn2;
                                    for(int m=0; m < updtn2; ++m){
                                        int cm =r+m;
                                        if(tempmatdistrib[cm] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                    }
                                    // delete the row of matinsg involving the sterile female
                                    tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );
                                    if(availfempos.empty()){
                                        comments+="\nIncomplete sample caused by female aging\n";

                                        break;
                                    }
                                }
                                else if(malesterile){

                                    /// check that not all remaining males are false
                                    bool mcheck=true;
                                    for(const auto mp: availmalepos ){
                                        mcheck*= !mp;
                                    }
                                    if(mcheck){
                                        comments+="\nIncomplete sample caused by male aging\n";
                                        break;
                                    }
                                    //AgeMale.erase(AgeMale.begin() + male);
                                   /// check if we need to update updtmaxpref
                                    for(int f=0; f < updtn1; ++f){
                                        int c = f*updtn2 + male;
                                        if(tempmatdistrib[c] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }
                                        // delete the col of matinsg involving the sterile female
                                        //tempmatdistrib.erase(tempmatdistrib.begin()+c);
                                    }

                                    if(availmalepos.empty()){
                                        comments+="\nIncomplete sample caused by male aging\n";

                                        break;
                                    }
                                }

                            } // close if(EM && !massEM)

                            if(femsterile || malesterile){ // some individual in the pair is sterile

                                ++attempts;

                                if(attempts > totpairs){
                                    comments="\nIncomplete sample\n";
                                    break;
                                }
                                else
                                    continue;
                            }

                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            double pij = matdistrib[couple]/ updtmaxpref;
                            if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
    //cout<<sumpref<<"\t"<<pij<<endl;
                            if(uniform(randgen) < pij ){ // mate
                                ++contmale; // for each male until the desired matings are performed
                                matfem[nummat] = tempfems[fem];
                                matmale[nummat] = tempmales[male];
                                ++nummat;

                                if(EM){
                                    if(!massEM){ // there is no replacement

                                        availfempos.erase(availfempos.begin()+fem);
                                        availmalepos.erase(availmalepos.begin()+male);
                                        tempfems.erase(tempfems.begin() + fem);
                                        tempmales.erase(tempmales.begin() + male);
                                        meanfemage += AgeFem[fem]; // sum here because is deleted
                                        AgeFem.erase(AgeFem.begin() + fem);
                                        meanmaleage += AgeMale[male];
                                        AgeMale.erase(AgeMale.begin() + male);

                                        /// check if we need to update updtmaxpref
                                        if(tempmatdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                        if(availfempos.empty() || availmalepos.empty()){
                                            comments+="\nIncomplete sample caused by aging\n";
                                            break;
                                        }

                                    }
                                    else{ // massEM model
                                        tempposfem.push_back(fem);
                                        tempposmale.push_back(male);
                                        if(matdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        if(attempts > maxattempts){
                                            int sampled = tempposfem.size();
                                            for(int a=0; a< sampled; ++a){
                                                int tfem= tempposfem[a];
                                                int tmale=tempposmale[a];
                                                availfempos[tfem]=false;
                                                availmalepos[tmale]=false;
                                            }
                                            updtmaxpref = updtmaxprefmass;
                                            attempts=0;
                                            tempposfem.clear();
                                            tempposmale.clear();
                                        }

                                    }
                                } // close if EM
                                if(impatient){ // it is ok here because if nummat does not increase the impatiente does not change

                                    p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                                }
                            } // close if u<pij

                            ++attempts;
                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";

                                break;
                            }

                        }while(nummat<nprime);

                    } // close if one choosy
else{
                    do{

                        bool malesterile=false;
                        bool femsterile=false;
                        int updtn1 = availfempos.size();
                        int updtn2 = availmalepos.size();

                        /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                        int updtn = updtn1*updtn2;
                        int couple;
                        int tries=0;

                        do{ // choose
                            couple = uniform(randgen)*updtn;
                            fem = couple/ updtn2;
                            male = couple % updtn2;
                            ++tries;
                            if(tries> totpairs){
                                if(!availfempos[fem])
                                    comments+="\nUnavailable females\n";
                                if(!availmalepos[male])
                                    comments+="\nUnavailable males\n";
//cout<<comments<<endl;
                                dobreak=true;
                                break;
                            }
                        }while(!availfempos[fem] || !availmalepos[male]);
//cout<<updtn<<"\t"<<updtcont<<"\t"<<updtmaxpref<<endl;
                        if(dobreak) break;

                /// Weibull aging process: we use log(age) units
                        ++AgeFem[fem];
                        ++AgeMale[male];

                        if(log(AgeFem[fem])>age_gamma){

                            double Rfem= exp(-pow((log(AgeFem[fem]) -age_gamma)/age_alpha, age_beta));

                            if(uniform(randgen)>= Rfem){ // has reached sterility
                                femsterile=true;
                                ++countfemsterility;
                                meansterilefemage+= AgeFem[fem];

                                if(!EM || massEM){ // because if EM and !massEM the row of this female is deleted and it is not necesarry to put to false
                                    availfempos[fem]=false;
                                }

                            }

                        }

    //cout<<"3\t"<<updtn1<<"\t"<<updtn2<<endl;
                        if(log(AgeMale[male])>age_gamma){

                            double Rmale= exp(-pow((log(AgeMale[male]) -age_gamma)/age_alpha, age_beta));

                            if(uniform(randgen)>= Rmale){ // has reached sterility
                                malesterile=true;
                                ++countmalesterility;
                                meansterilemaleage+= AgeMale[male];
                                availmalepos[male]=false;

                            }
                        }

                        /// Deal with specifities of the EM model
                        if(EM && !massEM){


                            if(femsterile && malesterile){ // the corresponding to fem*updtn2 + male must be deleted only once

                                /// female
                                availfempos.erase(availfempos.begin() + fem);
                                tempfems.erase(tempfems.begin() + fem);

                                AgeFem.erase(AgeFem.begin() + fem);

                               /// check if we need to update updtmaxpref
                                int r = fem*updtn2 ;
                                for(int m=0; m < updtn2; ++m){
                                    int cm =r+m;
                                    if(tempmatdistrib[cm] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                }
                                // delete the row of matinsg involving the sterile female
                                tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );
                                if(availfempos.empty()){
                                    comments+="\nIncomplete sample caused by female aging\n";

                                    break;
                                }

                                /// male

                                /// check that not all remaining males are false
                                bool mcheck=true;
                                for(const auto mp: availmalepos){
                                    mcheck*= !mp;
                                }
                                if(mcheck){
                                    comments+="\nIncomplete sample caused by male aging\n";
                                    break;
                                }
                                //AgeMale.erase(AgeMale.begin() + male);
                               /// check if we need to update updtmaxpref. ***Note here we use availfempos.size() instead updtn1 because we already erased some fems
                                int availfemposSize=availfempos.size();
                                for(int f=0; f < availfemposSize; ++f){
                                    if(f == fem) continue; /// ignore this case because it was already deleted
                                    int c = f*updtn2 + male;
                                    if(tempmatdistrib[c] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                }

                                if(availmalepos.empty()){
                                    comments+="\nIncomplete sample caused by male aging\n";

                                    break;
                                }

                            }
                            else if(femsterile){

                                availfempos.erase(availfempos.begin() + fem);
                                tempfems.erase(tempfems.begin() + fem);

                                AgeFem.erase(AgeFem.begin() + fem);

                               /// check if we need to update updtmaxpref
                                int r = fem*updtn2;
                                for(int m=0; m < updtn2; ++m){
                                    int cm =r+m;
                                    if(tempmatdistrib[cm] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                }
                                // delete the row of matinsg involving the sterile female
                                tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );
                                if(availfempos.empty()){
                                    comments+="\nIncomplete sample caused by female aging\n";

                                    break;
                                }
                            }
                            else if(malesterile){

                                /// check that not all remaining males are false
                                bool mcheck=true;
                                for(const auto mp: availmalepos ){
                                    mcheck*= !mp;
                                }
                                if(mcheck){
                                    comments+="\nIncomplete sample caused by male aging\n";
                                    break;
                                }
                                //AgeMale.erase(AgeMale.begin() + male);
                               /// check if we need to update updtmaxpref
                                for(int f=0; f < updtn1; ++f){
                                    int c = f*updtn2 + male;
                                    if(tempmatdistrib[c] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }
                                    // delete the col of matinsg involving the sterile female
                                    //tempmatdistrib.erase(tempmatdistrib.begin()+c);
                                }

                                if(availmalepos.empty()){
                                    comments+="\nIncomplete sample caused by male aging\n";

                                    break;
                                }
                            }

                        } // close if(EM && !massEM)

                        if(femsterile || malesterile){ // some individual in the pair is sterile

                            ++attempts;

                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";
                                break;
                            }
                            else
                                continue;
                        }

                        /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                        double pij = matdistrib[couple]/ updtmaxpref;
                        if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
//cout<<sumpref<<"\t"<<pij<<endl;
                        if(uniform(randgen) < pij ){ // mate

                            matfem[nummat] = tempfems[fem];
                            matmale[nummat] = tempmales[male];
                            ++nummat;

                            if(EM){
                                if(!massEM){ // there is no replacement

                                    availfempos.erase(availfempos.begin()+fem);
                                    availmalepos.erase(availmalepos.begin()+male);
                                    tempfems.erase(tempfems.begin() + fem);
                                    tempmales.erase(tempmales.begin() + male);
                                    meanfemage += AgeFem[fem]; // sum here because is deleted
                                    AgeFem.erase(AgeFem.begin() + fem);
                                    meanmaleage += AgeMale[male];
                                    AgeMale.erase(AgeMale.begin() + male);

                                    /// check if we need to update updtmaxpref
                                    if(tempmatdistrib[couple] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                    tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                    if(availfempos.empty() || availmalepos.empty()){
                                        comments+="\nIncomplete sample caused by aging\n";
                                        break;
                                    }

                                }
                                else{ // massEM model
                                    tempposfem.push_back(fem);
                                    tempposmale.push_back(male);
                                    if(matdistrib[couple] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                    if(attempts > maxattempts){
                                        int sampled = tempposfem.size();
                                        for(int a=0; a< sampled; ++a){
                                            int tfem= tempposfem[a];
                                            int tmale=tempposmale[a];
                                            availfempos[tfem]=false;
                                            availmalepos[tmale]=false;
                                        }
                                        updtmaxpref = updtmaxprefmass;
                                        attempts=0;
                                        tempposfem.clear();
                                        tempposmale.clear();
                                    }

                                }
                            } // close if EM
                            if(impatient){ // it is ok here because if nummat does not increase the impatiente does not change

                                p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                            }
                        } // close if u<pij

                        ++attempts;
                        if(attempts > totpairs){
                            comments="\nIncomplete sample\n";

                            break;
                        }

                    }while(nummat<nprime);
} // close else if onechoosy
                    realizedsamplesize[nf]=nummat;

                    if(bimodal)
                        ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                    else
                        ofilename+=NtoS(nf+1+nameshift)+".txt";
                    comments+="Positive assortative Gaussian preference";
                    if(writefiles)
                        writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                    else
                        cout<< "\nFile "<<nf+1<<" done"<<endl;
                    pair<double,double> regression;
                    if(nummat>4)
                        correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                    else
                        cout<<"no enough matings in file "<< nf+1<<endl;

                    filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";


                    /// reset
                    ofilename=dir+filename; // reset the base file name

                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
    // reset the matings vector
                    std::fill(matfem.begin(), matfem.end(), missing_value);
                    std::fill(matmale.begin(), matmale.end(), missing_value);

                    meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                    meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
                }// close for numfiles
                meanfemage/=(nummat*numfiles);
                meanmaleage/= (nummat*numfiles);
            } // close if choice > 0
            else if(choice<0){

            /// Compute the whole population mating distribution
                deque<double> matdistrib(totpairs,0);

                for(int f=0;f<n1;++f){
                    int row = f*n2;
                    for(int m=0; m<n2; ++m){
                        p[2]= (fabs(availfemales[f]-availmales[m])-Dmax);
                        p[2]*=p[2];
                        matdistrib[row + m] = computepropensity(p, Gaussbasedpref);
                    }
                }

                deque<double> sortmatdistrib = matdistrib;
                sort(sortmatdistrib.begin(), sortmatdistrib.end());
                auto indxmaxpref = max_element(matdistrib.begin(), matdistrib.end());
                double maxpref= *indxmaxpref;
                //int posmaxpref = distance(matdistrib.begin(), indxmaxpref);

                assert(matdistrib[distance(matdistrib.begin(), indxmaxpref)]== sortmatdistrib[totpairs-1]);

                for(int nf=0; nf< numfiles; ++nf){


                    vector<int> AgeFem(n1,0), AgeMale(n2,0);
                    // this are necessary because the aging process

/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
                    deque<double> tempmatdistrib;
                    if(EM && !massEM){
                        tempmatdistrib =matdistrib;
                    }
                    vector<double> tempfems = availfemales;
                    vector<double> tempmales = availmales;
                    vector<bool>availfempos(n1,true);
                    vector<bool>availmalepos(n2,true);
                    vector<int>tempposfem, tempposmale;
                    tempposfem.reserve(n1);
                    tempposmale.reserve(n2);
                    int fem=-1, male=-1;
                    int attempts=0;
                    nummat=0;
                    comments="";
                    double updtmaxpref = maxpref;
                    double updtmaxprefmass=updtmaxpref;

                    int updtcont=0;
                    bool dobreak=false;
                    int countmalesterility =0,countfemsterility=0;
                    if(onechoosy){ // only one sex chooses so this sex always mate: for a given male j the position in matdistrib is matdistrib[fem*n2+j]
                        int contmale=0;
                        do{
                            bool malesterile=false;
                            bool femsterile=false;

                            int updtn1 = availfempos.size();
                            int updtn2 = availmalepos.size();
                            //int updtn = updtn1*updtn2; // choose randomly a couple in the matdistrib matrix
                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012

                            int tries=0;

                            male=contmale%updtn2; // if nprime is higher than the number of males so contmale can be higher than the number of males we reset to the first male and repeat

                            while(!availmalepos[male]){
                                ++contmale;
                                if(tries> totpairs){

                                    comments+="\nUnavailable males\n";
    //cout<<comments<<endl;
                                    dobreak=true;
                                    break;
                                }
                                male=contmale%updtn2;
                            }
                            if(dobreak) break;
                            do{ // choose a female
                                fem = uniform(randgen)*updtn1; // choose randomly a female
                                //fem = couple/ updtn2;
                                ++tries;
                                if(tries> totpairs){
                                    if(!availfempos[fem])
                                        comments+="\nUnavailable females\n";
    //cout<<comments<<endl;
                                    dobreak=true;
                                    break;
                                }

                            }while(!availfempos[fem]);

                            if(dobreak) break;



                    /// Weibull aging process: we use log(age) units
                            ++AgeFem[fem];
                            ++AgeMale[male];
                            int couple= fem*updtn2+male;
                            if(log(AgeFem[fem])>age_gamma){

                                double Rfem= exp(-pow((log(AgeFem[fem]) -age_gamma)/age_alpha, age_beta));

                                if(uniform(randgen)>= Rfem){ // has reached sterility
                                    femsterile=true;
                                    ++countfemsterility;
                                    meansterilefemage+= AgeFem[fem];
                                    if(!EM || massEM){ // because if EM && !massEM we will delete this position
                                        availfempos[fem]=false;
                                    }

                                }

                            }

        //cout<<"3\t"<<updtn1<<"\t"<<updtn2<<endl;
                            if(log(AgeMale[male])>age_gamma){

                                double Rmale= exp(-pow((log(AgeMale[male]) -age_gamma)/age_alpha, age_beta));

                                if(uniform(randgen)>= Rmale){ // has reached sterility
                                    malesterile=true;
                                    ++countmalesterility;
                                    meansterilemaleage+= AgeMale[male];
                                    availmalepos[male]=false;

                                }
                            }

                            /// Deal with specifities of the EM model
                            if(EM && !massEM){

                                if(femsterile && malesterile){ // the corresponding to fem*updtn2 + male must be deleted only once

                                    availfempos.erase(availfempos.begin() + fem);
                                    tempfems.erase(tempfems.begin() + fem);

                                    AgeFem.erase(AgeFem.begin() + fem);
                                   /// check if we need to update updtmaxpref
                                    int r = fem*updtn2;
                                    for(int m=0; m < updtn2; ++m){
                                        int cm =r+m;
                                        if(tempmatdistrib[cm] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }
                                    }
                                    // delete the row of matinsg involving the sterile female
                                    tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );

                                    if(availfempos.empty()){
                                        comments+="\nIncomplete sample caused by female aging\n";

                                        break;
                                    }

                                    /// male

                                    /// check that not all remaining males are false
                                    bool mcheck=true;
                                    for(const auto mp: availmalepos){
                                        mcheck*= !mp;
                                    }
                                    if(mcheck){
                                        comments+="\nIncomplete sample caused by male aging\n";
                                        break;
                                    }

                                   /// check if we need to update updtmaxpref. ***Note here we use availfempos.size() instead updtn1 because we already erased some fems
                                    int availfemposSize=availfempos.size();
                                    for(int f=0; f < availfemposSize; ++f){
                                        if(f == fem) continue; /// ignore this case because it was already deleted
                                        int c = f*updtn2 + male;
                                        if(tempmatdistrib[c] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                    }

                                    if(availmalepos.empty()){
                                        comments+="\nIncomplete sample caused by male aging\n";

                                        break;
                                    }

                                }
                                else if(femsterile){

                                    availfempos.erase(availfempos.begin() + fem);
                                    tempfems.erase(tempfems.begin() + fem);

                                    AgeFem.erase(AgeFem.begin() + fem);
                                   /// check if we need to update updtmaxpref
                                    int r = fem*updtn2;
                                    for(int m=0; m < updtn2; ++m){
                                        int cm =r+m;
                                        if(tempmatdistrib[cm] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }
                                    }
                                    // delete the row of matinsg involving the sterile female
                                    tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );

                                    if(availfempos.empty()){
                                        comments+="\nIncomplete sample caused by female aging\n";

                                        break;
                                    }
                                }
                                else if(malesterile){

                                    /// check that not all remaining males are false
                                    bool mcheck=true;
                                    for(const auto mp: availmalepos){
                                        mcheck*= !mp;
                                    }
                                    if(mcheck){
                                        comments+="\nIncomplete sample caused by male aging\n";
                                        break;
                                    }
                                   /// check if we need to update updtmaxpref

                                    for(int f=0; f < updtn1; ++f){
                                        int c = f*updtn2 + male;
                                        if(tempmatdistrib[c] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                    }
                                    if(availmalepos.empty()){
                                        comments+="\nIncomplete sample caused by male aging\n";

                                        break;
                                    }
                                }

                            } // close if(EM && !massEM)

                            if(femsterile || malesterile){ // some individual in the pair is sterile

                                ++attempts;

                                if(attempts > totpairs){
                                    comments="\nIncomplete sample\n";
                                    break;
                                }
                                else
                                    continue;
                            }


                            /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                            double pij = matdistrib[couple]/ updtmaxpref;
                            if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
    //cout<<tempfems[fem]<<"\t"<<tempmales[male]<<"\t"<<updtmaxpref<<"\t"<<pij<<endl;
                            if(uniform(randgen) < pij ){ // mate
                                ++contmale; // for each male until the desired matings are performed
                                matfem[nummat] = tempfems[fem];
                                matmale[nummat] = tempmales[male];
                                ++nummat;

                                if(EM){
                                    if(!massEM){ // there is no replacement

                                        availfempos.erase(availfempos.begin()+fem);
                                        tempfems.erase(tempfems.begin() + fem);
                                        meanfemage += AgeFem[fem];
                                        AgeFem.erase(AgeFem.begin() + fem);

                                        availmalepos.erase(availmalepos.begin()+male);
                                        tempmales.erase(tempmales.begin() + male);
                                        meanmaleage += AgeMale[male];
                                        AgeMale.erase(AgeMale.begin() + male);

                                        /// check if we need to update updtmaxpref
                                        if(tempmatdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                        if(availfempos.empty() || availmalepos.empty()){
                                            if(nummat<nprime)
                                                comments+="\nIncomplete sample\n";
                                            break;
                                        }

                                    }
                                    else{ // massEM model
                                        tempposfem.push_back(fem);
                                        tempposmale.push_back(male);
                                        if(matdistrib[couple] == updtmaxpref){
                                            ++updtcont;
                                            updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                        }

                                        if(attempts > maxattempts){
                                            int sampled = tempposfem.size();
                                            for(int a=0; a< sampled; ++a){
                                                int tfem= tempposfem[a];
                                                int tmale=tempposmale[a];
                                                availfempos[tfem]=false;
                                                availmalepos[tmale]=false;
                                            }
                                            updtmaxpref = updtmaxprefmass;
                                            attempts=0;
                                            tempposfem.clear();
                                            tempposmale.clear();
                                        }

                                    }
                                } // close if EM

                                if(impatient){
                                    p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                                }
                            } // close if u<pij

                            ++attempts;
                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";

                                break;
                            }

//cout<<attempts<<"\t"<<nummat<<endl;
                        }while(nummat<nprime);

                    } // close if one choosy
else{
                    do{

                        bool malesterile=false;
                        bool femsterile=false;
                        int updtn1 = availfempos.size();
                        int updtn2 = availmalepos.size();

                        /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                        int updtn = updtn1*updtn2;
                        int couple;
                        int tries=0;

                        do{ // choose a female
                            couple = uniform(randgen)*updtn;
                            fem = couple/ updtn2;
                            male = couple % updtn2;
                            ++tries;
                            if(tries> totpairs){
                                if(!availfempos[fem])
                                    comments+="\nUnavailable females\n";
                                if(!availmalepos[male])
                                    comments+="\nUnavailable males\n";
//cout<<comments<<endl;
                                dobreak=true;
                                break;
                            }
                        }while(!availfempos[fem] || !availmalepos[male]);

                        if(dobreak) break;

                /// Weibull aging process: we use log(age) units
                        ++AgeFem[fem];
                        ++AgeMale[male];

//cout<<fem<<"\t"<<male<<endl;
//cout<<updtn1<<"\t"<<updtn2<<"\t"<<updtn<<"\t"<<"\t"<<AgeFem[fem]<<"\t"<<AgeMale[male]<<endl;


                        if(log(AgeFem[fem])>age_gamma){

                            double Rfem= exp(-pow((log(AgeFem[fem]) -age_gamma)/age_alpha, age_beta));

                            if(uniform(randgen)>= Rfem){ // has reached sterility
                                femsterile=true;
                                ++countfemsterility;
                                meansterilefemage+= AgeFem[fem];
                                if(!EM || massEM){ // because if EM && !massEM we will delete this position
                                    availfempos[fem]=false;
                                }

                            }

                        }

    //cout<<"3\t"<<updtn1<<"\t"<<updtn2<<endl;
                        if(log(AgeMale[male])>age_gamma){

                            double Rmale= exp(-pow((log(AgeMale[male]) -age_gamma)/age_alpha, age_beta));

                            if(uniform(randgen)>= Rmale){ // has reached sterility
                                malesterile=true;
                                ++countmalesterility;
                                meansterilemaleage+= AgeMale[male];
                                availmalepos[male]=false;

                            }
                        }

                        /// Deal with specifities of the EM model
                        if(EM && !massEM){


                            if(femsterile && malesterile){ // the corresponding to fem*updtn2 + male must be deleted only once

                                availfempos.erase(availfempos.begin() + fem);
                                tempfems.erase(tempfems.begin() + fem);

                                AgeFem.erase(AgeFem.begin() + fem);
                               /// check if we need to update updtmaxpref
                                int r = fem*updtn2;
                                for(int m=0; m < updtn2; ++m){
                                    int cm =r+m;
                                    if(tempmatdistrib[cm] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }
                                }
                                // delete the row of matinsg involving the sterile female
                                tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );

                                if(availfempos.empty()){
                                    comments+="\nIncomplete sample caused by female aging\n";

                                    break;
                                }

                                /// male

                                /// check that not all remaining males are false
                                bool mcheck=true;
                                for(const auto mp: availmalepos){
                                    mcheck*= !mp;
                                }
                                if(mcheck){
                                    comments+="\nIncomplete sample caused by male aging\n";
                                    break;
                                }

                               /// check if we need to update updtmaxpref. ***Note here we use availfempos.size() instead updtn1 because we already erased some fems
                                int availfemposSize=availfempos.size();
                                for(int f=0; f < availfemposSize; ++f){
                                    if(f == fem) continue; /// ignore this case because it was already deleted
                                    int c = f*updtn2 + male;
                                    if(tempmatdistrib[c] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                }

                                if(availmalepos.empty()){
                                    comments+="\nIncomplete sample caused by male aging\n";

                                    break;
                                }

                            }
                            else if(femsterile){

                                availfempos.erase(availfempos.begin() + fem);
                                tempfems.erase(tempfems.begin() + fem);

                                AgeFem.erase(AgeFem.begin() + fem);
                               /// check if we need to update updtmaxpref
                                int r = fem*updtn2;
                                for(int m=0; m < updtn2; ++m){
                                    int cm =r+m;
                                    if(tempmatdistrib[cm] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }
                                }
                                // delete the row of matinsg involving the sterile female
                                tempmatdistrib.erase(tempmatdistrib.begin()+r,tempmatdistrib.begin()+r+updtn2 );

                                if(availfempos.empty()){
                                    comments+="\nIncomplete sample caused by female aging\n";

                                    break;
                                }
                            }
                            else if(malesterile){

                                /// check that not all remaining males are false
                                bool mcheck=true;
                                for(const auto mp: availmalepos){
                                    mcheck*= !mp;
                                }
                                if(mcheck){
                                    comments+="\nIncomplete sample caused by male aging\n";
                                    break;
                                }
                               /// check if we need to update updtmaxpref

                                for(int f=0; f < updtn1; ++f){
                                    int c = f*updtn2 + male;
                                    if(tempmatdistrib[c] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                }
                                if(availmalepos.empty()){
                                    comments+="\nIncomplete sample caused by male aging\n";

                                    break;
                                }
                            }

                        } // close if(EM && !massEM)

                        if(femsterile || malesterile){ // some individual in the pair is sterile

                            ++attempts;

                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";
                                break;
                            }
                            else
                                continue;
                        }


                        /// Stochastic acceptance algorithm Lipowski and Lipowska 2012
                        double pij = matdistrib[couple]/ updtmaxpref;
                        if(EM && !massEM) pij = tempmatdistrib[couple]/ updtmaxpref;
//cout<<tempfems[fem]<<"\t"<<tempmales[male]<<"\t"<<updtmaxpref<<"\t"<<pij<<endl;
                        if(uniform(randgen) < pij ){ // mate

                            matfem[nummat] = tempfems[fem];
                            matmale[nummat] = tempmales[male];
                            ++nummat;

                            if(EM){
                                if(!massEM){ // there is no replacement

                                    availfempos.erase(availfempos.begin()+fem);
                                    tempfems.erase(tempfems.begin() + fem);
                                    meanfemage += AgeFem[fem];
                                    AgeFem.erase(AgeFem.begin() + fem);

                                    availmalepos.erase(availmalepos.begin()+male);
                                    tempmales.erase(tempmales.begin() + male);
                                    meanmaleage += AgeMale[male];
                                    AgeMale.erase(AgeMale.begin() + male);

                                    /// check if we need to update updtmaxpref
                                    if(tempmatdistrib[couple] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxpref= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                    tempmatdistrib.erase(tempmatdistrib.begin()+couple);

                                    if(availfempos.empty() || availmalepos.empty()){
                                        if(nummat<nprime)
                                            comments+="\nIncomplete sample\n";
                                        break;
                                    }

                                }
                                else{ // massEM model
                                    tempposfem.push_back(fem);
                                    tempposmale.push_back(male);
                                    if(matdistrib[couple] == updtmaxpref){
                                        ++updtcont;
                                        updtmaxprefmass= sortmatdistrib[totpairs-1-updtcont];
                                    }

                                    if(attempts > maxattempts){
                                        int sampled = tempposfem.size();
                                        for(int a=0; a< sampled; ++a){
                                            int tfem= tempposfem[a];
                                            int tmale=tempposmale[a];
                                            availfempos[tfem]=false;
                                            availmalepos[tmale]=false;
                                        }
                                        updtmaxpref = updtmaxprefmass;
                                        attempts=0;
                                        tempposfem.clear();
                                        tempposmale.clear();
                                    }

                                }
                            } // close if EM

                            if(impatient){
                                p[1]= fC /(nummat); // there is no necessary to put nummat+1 because the increase was already done
                            }
                        } // close if u<pij

                        ++attempts;
                        if(attempts > totpairs){
                            comments="\nIncomplete sample\n";

                            break;
                        }

//cout<<attempts<<"\t"<<nummat<<endl;

                    }while(nummat<nprime);
} // close else if choosy
                    realizedsamplesize[nf]=nummat;
                    if(bimodal)
                        ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                    else
                        ofilename+=NtoS(nf+1+nameshift)+".txt";
                    comments+="Negative assortative Gaussian preference";
                    if(writefiles)
                        writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                    else
                        cout<< "\nFile "<<nf+1<<" done"<<endl;
                    pair<double,double> regression;
                    if(nummat>4)
                        correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                    else
                        cout<<"no enough matings in file "<< nf+1<<endl;

                    filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";

                    /// reset
                    ofilename=dir+filename; // reset the base file name
                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
    // reset the matings vector
                    std::fill(matfem.begin(), matfem.end(), missing_value);
                    std::fill(matmale.begin(), matmale.end(), missing_value);
                    meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                    meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
                }// close for numfiles
                meanfemage/=(nummat*numfiles);
                meanmaleage/= (nummat*numfiles);
            } // close if choice < 0
            else{ // no preference

            /// Mating is random but still the encounter model could be different

                for(int nf=0; nf< numfiles; ++nf){

                    vector<int> AgeFem(n1,0), AgeMale(n2,0);
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed

                    vector<bool>availfempos(n1,true);
                    vector<bool>availmalepos(n2,true);
                    vector<int>tempposfem, tempposmale;
                    tempposfem.reserve(n1);
                    tempposmale.reserve(n2);
                    int fem=-1, male=-1;
                    int attempts=0;
                    nummat=0;
                    comments="";
                    int countmalesterility =0,countfemsterility=0;
                    do{
                        int updtn1 = availfempos.size();
                        int updtn2 = availmalepos.size();
                        do{ // choose a female
                            fem = uniform(randgen)*updtn1;
                        }while(!availfempos[fem]);

                        do{ // choose a male
                            male = uniform(randgen)*updtn2;
                        }while(!availmalepos[male]);

                /// Weibull aging process
                        ++AgeFem[fem];
                        ++AgeMale[male];

                        if(log(AgeFem[fem])>age_gamma){

                            double Rfem= exp(-pow((log(AgeFem[fem]) -age_gamma)/age_alpha, age_beta));

                            if(uniform(randgen)>= Rfem){ // has reached sterility
                                availfempos[fem]=false;

                                ++countfemsterility;
                                meansterilefemage+= AgeFem[fem];

                            }


                        }

                        if(log(AgeMale[male])>age_gamma){

                            double Rmale= exp(-pow((log(AgeMale[male]) -age_gamma)/age_alpha, age_beta));

                            if(uniform(randgen)>= Rmale){ // has reached sterility
                                availmalepos[male]=false;
                                ++countmalesterility;
                                meansterilemaleage+= AgeMale[male];
                            }
                        }

                        if(!availfempos[fem] || !availmalepos[male]){ // some individual in the pair is sterile

                            ++attempts;

                            if(attempts > totpairs){
                                comments="\nIncomplete sample\n";
                                break;
                            }
                            else
                                continue;
                        }

                         // mate
                        matfem[nummat] =availfemales[fem];
                        matmale[nummat] = availmales[male];
                        ++nummat;
                        if(EM){
                            if(!massEM){ // there is no replacement
                                availfempos[fem]=false;
                                availmalepos[male]=false;
                            }
                            else{ // massEM model
                                tempposfem.push_back(fem);
                                tempposmale.push_back(male);

                                if(attempts > maxattempts){
                                    int sampled = tempposfem.size();
                                    for(int a=0; a< sampled; ++a){
                                        int tfem= tempposfem[a];
                                        int tmale=tempposmale[a];
                                        availfempos[tfem]=false;
                                        availmalepos[tmale]=false;
                                    }
                                    attempts=0;
                                    tempposfem.clear();
                                    tempposmale.clear();
                                }

                            }
                        } // close if EM


                        ++attempts;

                        if(attempts > totpairs){
                            comments="\nIncomplete sample\n";
                            break;
                        }

                    }while(nummat<nprime);

                    realizedsamplesize[nf]=nummat;
                    if(bimodal)
                        ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                    else
                        ofilename+=NtoS(nf+1+nameshift)+".txt";
                    comments+="Random mating Gaussian preference";
                    if(writefiles)
                        writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                    else
                        cout<< "\nFile "<<nf+1<<" done"<<endl;
                    pair<double,double> regression;
                    if(nummat>4)
                        correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                    else
                        cout<<"no enough matings in file "<< nf+1<<endl;

                    filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";

                    /// reset
                    ofilename=dir+filename; // reset the base file name
                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
                        // reset the matings vector
                    std::fill(matfem.begin(), matfem.end(), missing_value);
                    std::fill(matmale.begin(), matmale.end(), missing_value);
                    meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                    meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
                }// close for numfiles
                meanfemage/=(nummat*numfiles);
                meanmaleage/= (nummat*numfiles);
            } // close choice = 0 i.e. random mating

        } // close gauss
        else{ /// Logistic model: logit pij = computepropensity(p, Logitpref);
            modeltype=3;
            filename+="Log_";
            filename+="a"+NtoS(logit_alpha0)+"_b"+NtoS(logit_alpha1)+"_";
            if(impatient) filename+="Imp_";
            ofilename=dir+filename;

            vector<double> p={logit_alpha0, logit_alpha1,trait,logit_alpha0,logit_alpha1, trait};

            for(int nf=0; nf< numfiles; ++nf){

                vector<int> AgeFem(n1,0), AgeMale(n2,0);
/// This copy is only used in the individual EM and permits to delete the pairs already matched from the temporal vector. This improves the speed
                vector<double> tempfems = availfemales;
                vector<double> tempmales = availmales;
                vector<bool>availfempos(n1,true);
                vector<bool>availmalepos(n2,true);
                vector<int>tempposfem, tempposmale;
                tempposfem.reserve(n1);
                tempposmale.reserve(n2);
                int fem=-1, male=-1;
                int attempts=0;
                nummat=0;
                comments="";

                int countmalesterility =0,countfemsterility=0;

                do{
                    bool sterile = false;
                    int updtn1 = availfempos.size();
                    int updtn2 = availmalepos.size();
//cout<<"1\t"<<updtn1<<"\t"<<updtn2<<endl;
                    do{ // choose a female
                        fem = uniform(randgen)*updtn1;
                    }while(!availfempos[fem]);

                    do{ // choose a male
                        male = uniform(randgen)*updtn2;
                    }while(!availmalepos[male]);
//cout<<"2\t"<<updtn1<<"\t"<<updtn2<<endl;
                /// Weibull aging process

                    ++AgeFem[fem];
                    ++AgeMale[male];

                    if(log(AgeFem[fem])>age_gamma){

                        double Rfem= exp(-pow((log(AgeFem[fem]) -age_gamma)/age_alpha, age_beta));

                        if(uniform(randgen)>= Rfem){ // has reached sterility

                            ++countfemsterility;
                            meansterilefemage+= AgeFem[fem];

                            if(EM && !massEM){

                                availfempos.erase(availfempos.begin() + fem);
                                tempfems.erase(tempfems.begin() + fem);

                                AgeFem.erase(AgeFem.begin() + fem);
                                if(availfempos.empty()){
                                    comments+="\nIncomplete sample caused by aging\n";
//cout<<"No fertile females"<<endl;
                                    break;
                                }
                            }
                            else{
                                availfempos[fem]=false;
                            }

                            sterile=true;
                        }
                    }

                    if(log(AgeMale[male])>age_gamma){

                        double Rmale= exp(-pow((log(AgeMale[male]) -age_gamma)/age_alpha, age_beta));

                        if(uniform(randgen)>= Rmale){ // has reached sterility

                            ++countmalesterility;
                            meansterilemaleage+= AgeMale[male];

                            if(EM && !massEM){
//cout<<"male "<<Rmale<<"\t"<<AgeMale[male]<<"\t"<<age_gamma<<endl;

                                availmalepos.erase(availmalepos.begin() + male);
                                tempmales.erase(tempmales.begin() + male);

                                AgeMale.erase(AgeMale.begin() + male);
                                if(availmalepos.empty()){
                                    comments+="\nIncomplete sample caused by aging\n";
//cout<<"No fertile males"<<endl;
                                    break;
                                }
                            }
                            else{
                                availmalepos[male]=false;
                            }

                            sterile=true;
                        }
                    }

//cout<<"4\t"<<updtn1<<"\t"<<updtn2<<endl;

                    if(sterile){ // some individual in the pair is sterile

                        ++attempts;

                        if(attempts > totpairs){
                            comments="\nIncomplete sample\n";
                            break;
                        }
                        else
                            continue;
                    }
                    else{
                        p[2] = tempmales[male]; // corresponds to the will of the female for the male trait
                        p[5] = tempfems[fem]; // will of the male for the female trait
                    }

                    /// pij
                    double pij = computepropensity(p, Logitpref);

                    if(uniform(randgen) < pij ){ // mate
                        matfem[nummat] =tempfems[fem];
                        matmale[nummat] = tempmales[male];
                        ++nummat;
                        if(EM){
                            if(!massEM){ // there is no replacement
//                                availfempos[fem]=false;
//                                availmalepos[male]=false;
                                availfempos.erase(availfempos.begin() + fem);
                                tempfems.erase(tempfems.begin() + fem);

                                meanfemage += AgeFem[fem];
                                AgeFem.erase(AgeFem.begin() + fem);

                                availmalepos.erase(availmalepos.begin() + male);
                                tempmales.erase(tempmales.begin() + male);

                                meanmaleage += AgeMale[male];
                                AgeMale.erase(AgeMale.begin() + male);

                                if(availfempos.empty() || availmalepos.empty()){
                                    comments+="\nIncomplete sample caused by aging\n";
                                    break;
                                }
                            }
                            else{ // massEM model
                                tempposfem.push_back(fem);
                                tempposmale.push_back(male);

                                if(attempts > maxattempts){
                                    int sampled = tempposfem.size();
                                    for(int a=0; a< sampled; ++a){
                                        int tfem= tempposfem[a];
                                        int tmale=tempposmale[a];
                                        availfempos[tfem]=false;
                                        availmalepos[tmale]=false;
                                    }
                                    attempts=0;
                                    tempposfem.clear();
                                    tempposmale.clear();
                                }

                            }
                        } // close if EM
                        if(impatient){
                            p[0]= logit_alpha0+ impatientfactor*nummat;
                            p[3]= logit_alpha0+ impatientfactor*nummat;
                        }
                    } // close if u<pij

                    ++attempts;

                    if(attempts > totpairs){
                        comments="\nIncomplete sample\n";

                        break;
                    }


                }while(nummat<nprime);

//cout<<"Num Sterile "<<countfemsterility<<"\t"<<countmalesterility<<endl;

                realizedsamplesize[nf]=nummat;
                if(bimodal)
                    ofilename+=NtoS(nf+1+nameshift)+"b.txt";
                else
                    ofilename+=NtoS(nf+1+nameshift)+".txt";
                comments+="logit preference";
                if(impatient) comments+=("_ImpatientFactor: "+NtoS(impatientfactor));
                if(writefiles)
                    writecontinuousmodel(ofilename,programname,comments,EM,massEM,n1,n2, matfem,  matmale,nummat);
                else
                    cout<< "\nFile "<<nf+1<<" done"<<endl;

                pair<double,double> regression;

                if(nummat>4)
                    correl[nf] = regcorrelxy(matfem, matmale,true,false, regression);
                else
                    cout<<"no enough matings in file "<< nf+1<<endl;

                filenames[nf] = filename+NtoS(nf+1+nameshift)+".txt";

                /// reset
                ofilename=dir+filename; // reset the base file name
                if(massEM){
                    tempposfem.clear();
                    tempposmale.clear();
                    vector<int>().swap(tempposfem);
                    vector<int>().swap(tempposmale);
                }

    // reset the matings vector
                std::fill(matfem.begin(), matfem.end(), missing_value);
                std::fill(matmale.begin(), matmale.end(), missing_value);


                meanfemage += accumulate(AgeFem.begin(), AgeFem.end(), 0);
                meanmaleage += accumulate(AgeMale.begin(), AgeMale.end(), 0);
            }// close for numfiles

            meanfemage/=(nummat*numfiles);
            meanmaleage/= (nummat*numfiles);

//cout<<"Mean Age "<<log(meanfemage)<<"\t"<<log(meanmaleage)<<endl;

        } // close logit

        // mean sampel size
        double meansamplesize= accumulate(realizedsamplesize.begin(), realizedsamplesize.end(),0.0 );
        meansamplesize/=numfiles;
        if(bimodal)
            ofilename+="bCorrel.txt";
        else
            ofilename+="Correl.txt";

        writecorrelations(ofilename,programname,comments,EM, massEM,n1, n2,meansamplesize, filenames, correl, (meanfemage), (meanmaleage));
} // close Age structure

/// If indicated by the user argument, write phenotypic distribution after scaling
if(writedistrib){
    if(verbose) cout<<"Writing phenotypic distribution\n";
    dir=dirname+"/";
    string fn = dir+"phenotypes.txt";

    writephenotypedistrib(fn,"MateSim",n1, n2, availfemales, availmales);
}


    } // close if not discrete
    else{ /// if(discrete)

    scaling=false;

/// *********************************************************************************
/// *************************** DISCRETE TRAITS *************************************
/// *********************************************************************************

    //bool onesmplround = R <=1; // if the number of matings rounds R is > 1 then there are more than one sample round and mmax = nprime/R. By default is mmax=nprime and R=1


    if(onesmplround){ // The sampling is performed in the last mating round independently if it is with or without replacement model l
        // abril 2019: if we desire random mating with mass encounter we set modeltype = -123321

        bool EM= modeltype < 0; // detect if the desired model is Encounter-mating type (without replacement models)
        modeltype = abs(modeltype); // detect if the model is with mass encounters
        bool massEM = (modeltype>9) && EM;
        int minNsex = min(n1,n2);
        if(massEM){

            if(mmax < 0){ // was not defined
                mmax = minNsex;

            }
            else
            if(mmax>minNsex) {

                string msg = "A mass-encounter model was defined so that the number of encounters per mating round "+NtoS(mmax)+ " cannot be higher than the number of individuals of one sex "+NtoS(minNsex) +"\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.err", msg,true,true);
            }

            if(modeltype != 123321)
                modeltype/=10;
            else
                modeltype =0; // now there is random mating under massencounter
        }


        filename+="MateSim_D_";

        if(EM){
            if(massEM){
                filename+="massEM_";
                if(mmax < minNsex){
                    filename+= NtoS(mmax)+"_";
                }
            }
            else
                filename+="iEM_";
        }

        if(modeltype<6){
            filename+="M"+NtoS(modeltype)+"_a";

            if(nullrow<0) // has not been settled by argument. For these models represent the default row with unitary propensity
                nullrow=k1-1;

            if(nullcol<0) // has not been settled by argument. For these models represent the default column with unitary propensity
                nullcol=k2-1;
        }
        else{ // Separated parameters 2019 discrete models
            filename+="M"+NtoS(modeltype)+"_ac";

            if(nullrow<0) // has not been settled by argument. For these models represent the default row with differential intrasexual competition
                nullrow=0;

            if(nullcol<0)// has not been settled by argument. For these models represent the default column with differential intrasexual competition
                nullcol=0;
        }

    /// Check the number of possible matings depending on the population and the pair formation process

        int maxmatings=n1*n2;

        if(EM){ // without replacement

            maxmatings= minNsex;

        }
        if(nprime > maxmatings){

            string msg = "\nToo few population males or females (" +NtoS(maxmatings)+ ") for too many matings (" +NtoS(nprime)+ ")\n";
            cout<<msg<<endl;
            errormsg(maxmatings,"MateSim.err", msg,true,true);

        }


        // vector for mutual propensities defining the type of model
        vector<vector<double>>m(k1,vector<double>(k2,1));
        double a=1, choice=1;

        switch(modeltype){

           case 0: // Neutral M0 models
    //           for(auto& mr:m)
    //            for(auto& mij: mr)
    //                mij=1;
               filename+=NtoS(a)+"_";
               break;

           case 1: // Female sexual selection models with uniform or different frequencies
                // all but the null row could have non-neutral propensity
                a =iniSSval*posfactor;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r!=nullrow)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_";

               break;

            // Male sexual selection models with uniform or different frequencies

           case 2: // Male sexual selection models with uniform or different frequencies
                // all but the null col could have non-neutral propensity
                a=iniSSval*posfactor;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(c!=nullcol)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_";
               break;

           case 3: // Positive assortative strict (uniform freqs) and mixed
                if(choicefactor<1) errormsg(choicefactor,"MateSim.err", "Wrong < 1 choice-factor for model 3",true,true);
                a= iniIsolval*choicefactor;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r==c)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_";
               break;
           case 4: // Negative assortative strict (uniform) and mixed
                if(choicefactor>1) errormsg(choicefactor,"MateSim.err", "Wrong > 1 choice-factor for model 4",true,true);
                a=iniIsolval*choicefactor; // choicefactor is between 1 and inf
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r==c)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_";
               break;
           case 5: // User defined of preferences
                {
                    string path=""; // error: jump to case label is generated in the next case if this variable is declared out of the block
                    string name="MateSimUserModel.txt";
                    // this coudl reset k1 and k2
                    ReadUserModels(path,name,  m, k1, k2);
                    a=0;
                    filename+="User_";

                    break;
                }
           case 6: // Female sexual selection models with parameter a (in nullrow) and positive or negative choice c because we allow here choicefactor<1
                a =iniSSval*posfactor;
                choice =iniIsolval*choicefactor;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r==nullrow)
                            m[r][c]=a;
                        else if(r==c){ // if diagonal not nullrow,nullrow
                            m[r][c]=choice;
                        }
                    }
                }
                 filename+=NtoS(a)+"-"+NtoS(choice)+"_";

               break;
           case 7: // Male sexual selection models with parameter a (in nullcol) and choice c
                a =iniSSval*posfactor;
                choice =iniIsolval*choicefactor; // use choice factor lower than 1 for negative assortative here
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(c==nullcol)
                            m[r][c]=a;
                        else if(r==c){ // if diagonal not nullcol,nullcol
                            m[r][c]=choice;
                        }
                    }
                }
                 filename+=NtoS(a)+"-"+NtoS(choice)+"_";

               break;
           case 8: // Female sexual selection models with compound parameter a (in nullrow) and choice c
                a =iniSSval*posfactor;
                choice =iniIsolval*choicefactor;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r==0 && c==0)
                            m[r][c]= a*choice;
                        else
                        if(r==nullrow)
                            m[r][c]=a;
                        else if(r==c){ // if diagonal not nullrow,nullrow
                            m[r][c]=choice;
                        }
                    }
                }
                 filename+=NtoS(a)+"-"+NtoS(choice)+"_";

               break;
           case 9: // Mle sexual selection models with compound parameter a (in nullcol) and choice c
                a =iniSSval*posfactor;
                choice =iniIsolval*choicefactor;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r==0 && c==0)
                            m[r][c]= a*choice;
                        else
                        if(c==nullcol)
                            m[r][c]=a;
                        else if(r==c){ // if diagonal not nullcol,nullcol
                            m[r][c]=choice;
                        }
                    }
                }
                 filename+=NtoS(a)+"-"+NtoS(choice)+"_";

               break;
// Define models with separated parameters for competition and mate choice
            default:
                string msg = "Wrong model number "+NtoS(modeltype)+ "\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.err", msg,true,true);
        } // close switch

        if(k1!=k2 && format==0) // update format
            format=2;

        /*
        ///  Dec 2019: Fix a bug generated when k1 or k2 = 1 because mpf must be 0 for ReadUserFreqss to work
        if(k1==1 || k2==1)
            MPF = 0;
*/
        if(EM){ // Encounter-Mating model Gimelfarb 88
            if(massEM){
                dirname+="_massEM";
                if(mmax < minNsex){
                    dirname+= NtoS(mmax);
                }
            }
            else dirname+="_iEM";
        }

        if(modeltype<5)
            dirname+= "_"+ NtoS(modeltype)+"_a"+NtoS(a);
        else if(modeltype>5)
            dirname+= "_"+ NtoS(modeltype)+"_ac"+NtoS(a)+"-"+NtoS(choice);
        else
            dirname+= "_"+ NtoS(modeltype)+"_aUser";

        if(drift && uniformfreqs){
            dirname+="_driftU_sample"+NtoS(nprime);
            filename+="driftU"+NtoS(n)+"_";

        }
        else
        if(drift && !uniformfreqs && !userdef){
                dirname+="_driftnonU_sample"+NtoS(nprime);
                filename+="driftnonU"+NtoS(n)+"_";
        }
        else
            if(uniformfreqs){
                dirname+="_U_sample"+NtoS(nprime);
                filename+="U"+NtoS(n)+"_";
            }
            else if(!userdef){
                 dirname+="_nonUsample"+NtoS(nprime);
                 filename+="nonU"+NtoS(n)+"_";
                }
                else
                if(drift && userdef){
                    dirname+="_driftUser_sample"+NtoS(nprime);
                    filename+="driftUser"+NtoS(n)+"_";
                }
                else{
                    dirname+="_User_sample"+NtoS(nprime);
                    filename+="User"+NtoS(n)+"_";
                }
        dirname+="_N"+NtoS(n);
        dirname+="_MPF"+NtoS(MPF);
        portablemkdir(dirname);
        dir=dirname+"/";
        string ofilename=dir+filename;

        /// Population frequencies and observed numbers of individuals of each type
        vector<double> p1(k1,0), p2(k2,0);

        vector<double> userp1, userp2;
        vector<double> obsfems(k1,0), obsmales(k2,0);

        if(userdef){ // User-defined frequencies (this is different of user-defined model)
            userp1.resize(k1,0);
            userp2.resize(k2,0);
            string path="";
            string name="MateSimUserFreqs.txt";
            ReadUserFreqs(path,name, k1, k2, userp1, userp2,MPF);
            int acum=0;
            p1=userp1;
            p2=userp2;
            obsfems=userp1;
            obsmales=userp2;
            //
            for(int f=0; f< (k1-1); ++f){
                obsfems[f]*= n1;
                obsfems[f]=round(obsfems[f]);
                acum+=obsfems[f];
            }
            obsfems[k1-1] = n1-acum;
            acum=0;
            for(int m=0; m< (k2-1); ++m){
                obsmales[m]*=n2;
                obsmales[m]=round(obsmales[m]);
                acum+=obsmales[m];
            }
            obsmales[k2-1] = n2-acum;

        }


        vector<vector<double>>matings(k1,vector<double>(k2,0));
        int nf=0;

    /// without drift
        if(!drift && userdef){
           if(numfiles>1 && !EM){
                numfiles=1;
                string msg = "The number of files has been changed to 1 because there is no drift, the process is with replacement and the frequencies are uniform\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.warning", msg,true,false);
            }

//for(int f=0; f< (k1); ++f){
//    cout<<(obsfems[f])<<endl;
//}
//for(int m=0; m< (k2); ++m){
//    cout<<(obsmales[m])<<endl;
//}


        }
        else
        if(!drift && uniformfreqs){ // if uniform they are not user-def

            if(numfiles>1 && !EM){
                numfiles=1;
                string msg = "The number of files has been changed to 1 because there is no drift, the process is with replacement and the frequencies are uniform\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.warning", msg,true,false);
            }
            double acum=0;
            // generate female population freqs
            for(int f=0; f< (k1-1); ++f ){
                obsfems[f]=(double)n1/(double)k1; // the distribution of the k1 female types is uniform over the n1 females
                acum+=obsfems[f];
            }
            obsfems[k1-1] = n1-acum;

            // population frequencies
            p1=obsfems;
            for(auto& f:p1)
                f/=n1;

            acum=0;
            // generate male population freqs
            for(int f=0; f< (k2-1); ++f ){
                obsmales[f]=(double)n2/(double)k2;
                acum+=obsmales[f];
            }

            obsmales[k2-1] = n2-acum;

            // population frequencies
            p2=obsmales;

            for(auto& f:p2){
                f/=n2;
            }

        } // close if no drift && uniformfreqs
        else // non drift + non-uniform freqs
        if(!drift && !uniformfreqs && !userdef){ // the frequencies are not uniform and has not been given by the user
                if(numfiles>1 && !EM){
                    numfiles=1;
                    string msg = "The number of files has been changed to 1 because there is no drift and the process is with replacement.\n";
                    cout<<msg<<endl;
                    errormsg(uniformfreqs,"MateSim.warning", msg,true,false);
                }
                int acum=0;
                // females
                if(k1>1){ /// Dec 2019
                    for(int f=0; f< (k1-1); ++f ){
                        if(acum==n1-1) break;
                        double u=uniform(randgen);
                        while(u<MPF || u> (1.0-MPF)){
                            u=uniform(randgen);
                        }
                        //int minn1 =max((int)(MPF*n1),2);
                        //obsfems[f]=max((int)(u*n1),minn1); // at least MPF*n1 female or a minimum of 2 of each type
                        obsfems[f]=round(u*(n1-acum));
                        acum+=obsfems[f];
                        if(acum>=n1){obsfems[f]-=(1+acum-n1); acum=n1-1;}
                    }
                    obsfems[k1-1] = n1-acum; // forces at least 1 female
                    /// Dec 2019 shuffle the frequencies because if not when k>2 the obs[0] is in average greater than the others
                    shuffle(std::begin(obsfems), std::end(obsfems), randgen);

                }
                else /// k1 = 1 Dec 2019
                    obsfems[0] = n1;

                p1=obsfems;

                for(auto& f:p1)
                    f/=n1;

                acum=0;
                // males
                if(k2>1){ /// Dec 2019
                    for(int f=0; f< (k2-1); ++f ){
                        if(acum==n2-1) break;
                        double u=uniform(randgen);
                        while(u<MPF || u> (1.0-MPF)){
                            u=uniform(randgen);
                        }
                        //int minn2 =max((int)(MPF*n2),2);
                        //obsmales[f]=max((int)(n2*u),minn2);
                        obsmales[f]=round((n2-acum)*u);
                        acum+=obsmales[f];

                        if(acum>=n2){obsmales[f]-=(1+acum-n2); acum=n2-1;} // at least one male in the last class
                    }
                    obsmales[k2-1]=n2-acum;
                    /// Dec 2019 shuffle the frequencies because if not when k > 2 the obs[0] is in average greater than the others
                    shuffle(std::begin(obsmales), std::end(obsmales), randgen);
                }
                else /// k2 = 1 Dec 2019
                    obsmales[0]=n2;

                p2=obsmales;
                for(auto& f:p2){
                    f/=n2;
                }

        } // close if no drift && not uniformfreqs

/// Repeat for numfiles

        do{ // drift here means that the values are different among the different files even with the same parametric values

/// with drift
            if(drift && uniformfreqs){
                double invk1=1.0/k1, invk2=1.0/k2;
                int acum=0;

                do{ // obtain n1 females distributed in k1 types. The highest n1 the more uniform the frequencies

                /* Roulette wheel algorithm in Lipowski and Lipowska 2012
                    int type = k1* uniform(randgen);
                    if(uniform(randgen) < p(type) select this type
                */
                    for(int f=0; f< k1; ++f ){
                        if(acum==n1) break;
                        double u=uniform(randgen);
                        if( (f)*(invk1)<= (u) && (u)< (f+1)*(invk1)){
                            ++obsfems[f];
                            ++acum;
                        }
                    }
                }while(acum<n1);

                p1=obsfems;

                for(auto& f:p1)
                    f/=n1;


                acum=0;
                do{ // obtain n2 males distributed in k2 types. The highest n2 the more uniform the frequencies
                    for(int f=0; f< k2; ++f ){
                        if(acum==n2) break;
                        double u=uniform(randgen);
                        if((f)*(invk2)<=(u) && (u) < (f+1)*(invk2)){
                            ++obsmales[f];
                            ++acum;
                        }
                    }
                }while(acum<n2);

                p2=obsmales;
                for(auto& f:p2){
                    f/=n2;
                }


            } // close if drift and uniforfreqs
            else
            if(drift && !uniformfreqs && !userdef){ // non-uniform freqs and nonuserdef

                int acum=0;
                if(k1>1){ /// Dec 2019
                    for(int f=0; f< (k1-1); ++f ){
                        if(acum==n1-1) break;
                        double u=uniform(randgen); // this is inside the for (not a roulette) because the desired frequencies are unknown so they are just left to be random guesses between 1 and n1-k1 (no class with 0 individuals)
                        while(u<MPF || u> (1.0-MPF)){
                            u=uniform(randgen);
                        }
                        //int minn1 =max((int)(MPF*n1),2);
                        //obsfems[f]=max((int)(u*n1),minn1);
                        obsfems[f]=round(u*(n1-acum));
                        acum+=obsfems[f];
                        if(acum>=n1){obsfems[f]-=(1+acum-n1); acum=n1-1;}
                    }
                    obsfems[k1-1] = n1-acum; // forces at least 1 female
                    /// Dec 2019 shuffle the frequencies because if not when k>2 the obs[0] is in average greater than the others
                    shuffle(std::begin(obsfems), std::end(obsfems), randgen);
                }
                else /// k1 = 1 Dec 19
                    obsfems[0] = n1;

                p1=obsfems;

                for(auto& f:p1)
                    f/=n1;

                acum=0;
                if(k2>1){ /// Dec 2019
                    for(int m=0; m< (k2-1); ++m ){
                        if(acum==n2-1) break;
                        double u=uniform(randgen);
                        while(u<MPF || u> (1.0-MPF)){
                            u=uniform(randgen);
                        }
                        //int minn2 =max((int)(MPF*n2),2);
                        //obsmales[f]=max((int)(n2*u),minn2);
                        obsmales[m]=round((n2-acum)*u);
                        acum+=obsmales[m];

                        if(acum>=n2){obsmales[m]-=(1+acum-n2); acum=n2-1;} // at least one male in the last class
                    }
                    obsmales[k2-1]=n2-acum;
                    /// Dec 2019 shuffle the frequencies because if not when k>2 the obs[0] is in average greater than the others
                    shuffle(std::begin(obsmales), std::end(obsmales), randgen);
                }
                else /// k2=1 dec 2019
                    obsmales[0]=n2;

                p2=obsmales;
                for(auto& m:p2){
                    m/=n2;
                }

            } // close drift + non-uniform frequencies
            else if(drift && userdef){ // drift + user defined frequencies

                int acum=0;

                do{ // obtain n1 females distributed in k1 types. The highest n1 the more resemble the observed to the user defined frequencies
                    // roulette
                    double acumfreq=0.0;
                    double u=uniform(randgen);
                    double prev=0;
                    for(int f=0; f< k1; ++f ){
                        acumfreq+=userp1[f];
                        if( (u>=prev) && (u<acumfreq) ){
                            ++obsfems[f];
                            ++acum;
                            break;
                        }
                        else{
                            prev=acumfreq;
                        }

                    }
                }while(acum<n1);

                p1=obsfems;

                for(auto& f:p1)
                    f/=n1;

                acum=0;
                do{ // obtain n1 males distributed in k2 types. The highest n2 the more resemble the observed to the user defined frequencies
                    // roulette
                    double acumfreq=0.0;
                    double u=uniform(randgen);
                    double prev=0;
                    for(int m=0; m< k2; ++m ){
                        acumfreq+=userp2[m];
                        if( (u>=prev) && (u<acumfreq) ){
                            ++obsmales[m];
                            ++acum;
                            break;
                        }
                        else{
                            prev=acumfreq;
                        }

                    }
                }while(acum<n2);

                p2=obsmales;
                for(auto& f:p2){
                    f/=n2;
                }

            } // close drift + userdef

        /// compute mean M
            double M=0;
            for(int r=0; r<(k1);++r){
                for(int c=0; c<k2; ++c){
                    M+=(p1[r]*p2[c]*m[r][c]);
                }
            }

/// Models

/*
Without replacement: the encounter-mating model (Gimelfarb 1988, Gn and Yilmaz 2017)
 *
 *     Within this scenario we use the urn scheme (Gimelfarb 1988) for the process of pair formation:
 *      1-A) Individual encounters
 *      For a total number of desired matings M the mating table is formed one by One. The algorithm is
 *         Given m=0; n1 females and n2 males, xi(0) females of type 1 and yj(0) males of type j.
 *       i) use a roulette for choosing a female i with probability proportional to the frequency pi(m), similarly for male j at frequency qj(m)
 *      ii) get a uniform number U(0,1).
 *      iii)
 *          if U<= PIij/PImean {++m; if(m<M){--n1; --n2; pi(m) = (xi(0)-mi) /(n1-m); qj(m) = (yj(0)-mj) /(n2-m); update PImean;} else break}
 *          loop to i);
 *
 *      1-B) Mass encounters
 *      Given first mating round r=0; n1 females and n2 males, xi(0) females of type 1 and yj(0) males of type j
 *      and let n = min(n1,n2) the maximum number of pairs that can be arranged in the population.
        The maximum number of encounters in each round is mmax. The process will be repeated until nprime matings are obtained
 *      i) numm=0; Compute PIij = qrij*mij and the mean PImean
        ii) make a roulette for selecting mmax females and males
 *      iii) For each of the pairs formed in i) get a uniform number U(0,1).
 *      iii)
 *          Repeat
            if U<= PIij/PImean {++obs[r][i][j], ++mr, ++numm;
            until mmax encounters ocurred. Then compute mr (<=mmax) stores the number of matings that occurred in this round.
            And mfri = sum (Obs[r][i][] and mmrj = sum (Obs[r][][j]
            then update: if(numm<nprime){n1 = n1-mr; --n2; p1ri(m) = (xi(r)-mri) /(n1); p2rj(m) = (yj(r)-mmrj) /(n2-mr)} else break}
 *
 *      iv) loop to ii)
*/
/// Generate matings for sample size nprime under different models: EM, massEM...
            const bool append=true;
            if(EM && !massEM){ // individual EM model: model 1 from Gimelfarb 1988
                // pairs are formed step by step
                vector<double>females(k1,0);
                vector<double>males(k2,0);
             // because there is  no replacement the frequencies are updated after encounters
                vector<double> p1_t(p1), p2_t(p2);

                int nfem=n1, nmale=n2;
                int cm=0; // count matings

                do{

//cout<<nfem<<"\t"<<nmale<<endl;
                    // get 1 female
                    int fem = roulette(p1_t,append, females ); // in females the females selected for mating are stored

                    // get 1 male
                    int male = roulette(p2_t,append, males );

                    if(uniform(randgen) <= m[fem][male]/M ){ // this pair successfully mate
                        ++cm;
                        ++matings[fem][male];
                        if(cm<nprime){
                        // update values
                            --nfem; --nmale;

                        // update frequencies

                            double p1acum=0;
                            for(int i=0; i< (k1-1); ++i){
                                p1_t[i] = (obsfems[i]-females[i]) /(nfem);
                                p1acum+=p1_t[i];
                            }
                            p1_t[k1-1] = 1.0 -p1acum;

                            double p2acum=0;
                            for(int j=0; j< (k2-1); ++j){
                                p2_t[j] = (obsmales[j]-males[j]) /(nmale);
                                p2acum+=p2_t[j];
                            }
                            p2_t[k2-1] = 1.0-p2acum;

                            // update the mean preference (PI(k) in formula 10 in Gimelfarb 88)
                            M=0;
                            for(int r=0; r<(k1);++r){
                                for(int c=0; c<k2; ++c){
                                    M+=(p1_t[r]*p2_t[c]*m[r][c]);

                                }
                            }
assert(M>0);
                        }
                        else{ // the nprime matings were already obtained
                            break;
                        }

                    }
                    else{ // the mate did not happen and the fem and male return to the urn
                        --females[fem];
                        --males[male];
                    }
//cout<<cm<<endl;
                }while(cm<nprime);
            } // close model EM && !massEM
            else if(EM && massEM){ /// abril 2019 actualiza el mtodo para igualarlo al de mass encounter con varias rondas de muestreo (este es mass encounter pero el muestreo ocurre al final)
                // pairs are formed in mass

                // because there is  no replacement the frequencies are updated after encounters
                vector<double> p1_t(p1), p2_t(p2);

                int nfem=n1, nmale=n2;
                //int minn = min(n1,n2);
                int cm=0; // count matings
                int encounters=0; // count encounters
                int maxencounters = n1*n2;
                bool exhausted= false;
                vector<double>availfems = obsfems;
                vector<double>availmales = obsmales;

                int fem ; // abril 2019 declared here to could be checked out of the do while
                int male;

                do{
                    // get minn females and males
                    int mmin=min(nfem,nmale);
                    int matR=min((int)mmax,mmin);
                    vector<int>femindexes((int)matR,-1);
                    vector<int>maleindexes((int)matR,-1);
                    roulette(matR,p1_t,availfems, femindexes ); // availablefems is updated within
                    roulette(matR,p2_t,availmales, maleindexes ); // availablemales is updated within

                    // perform the matings
                    int contmr=0; // count mating by round

                    for(int mt=0; mt<matR; ++mt){

                        fem = femindexes[mt];
                        male = maleindexes[mt];

                        if(uniform(randgen) <= m[fem][male]/M ){ // this pair successfully mate

                            ++cm;
                            ++matings[fem][male];
                            ++contmr;

                            if(cm<nprime){
                            // update number of available females and males
                                --nfem; --nmale;

                            }
                            else{ // the nprime matings were already obtained
                                break;
                            }

                        }
                        else{ // the mate did not happen and the fem and male return to the urn
                            ++availfems[fem];
                            ++availmales[male];
                        }

                        ++encounters;

                    } // close for mt

                    if(encounters == maxencounters){
                        exhausted = true;

                    }

                    /// update after the mass encounter provided that the sample size hs not been reached and that some mating occurred

                    if(cm<nprime && contmr){

                        // update frequencies

                        double p1acum=0;
                        for(int i=0; i< (k1-1); ++i){
                            p1_t[i] = (availfems[i]) /(nfem);
                            p1acum+=p1_t[i];
                        }
                        p1_t[k1-1] = 1.0 -p1acum;

                        double p2acum=0;
                        for(int j=0; j< (k2-1); ++j){
                            p2_t[j] = (availmales[j]) /(nmale);
                            p2acum+=p2_t[j];
                        }
                        p2_t[k2-1] = 1.0-p2acum;

                    // update the mean M
                        M=0;
                        for(int r=0; r<(k1);++r){
                            for(int c=0; c<k2; ++c){
                                M+=(p1_t[r]*p2_t[c]*m[r][c]);


//if(M<0){
//    cout<<r<<"\t"<<p1acum<<c<<"\t"<<p2acum<<endl;
//    cout<<nfem<<"\t"<<nmale<<"\t"<<p1_t[r]<<"\t"<<p2_t[c]<<"\t"<<m[r][c]<<"\t"<<M<<endl;
//    for(auto & f:obsfems)
//        cout<<f<<"\t";
//    cout<<endl;
//    for(auto & f:availfems)
//        cout<<f<<"\t";
//    cout<<endl;
//    for(auto & m:availmales)
//        cout<<m<<"\t";
//    cout<<endl;
//    for(auto & m:obsmales)
//        cout<<m<<"\t";
//    exit(1);
//}
                            }
                        }
assert(M>0);
                    } // close if cm < nprime



                }while(cm<nprime && !exhausted);

                if(exhausted){
                    if(verbose){
                        cout<< "File " << nf+2 <<" incomplete"<<endl; // it is +1 because nf for this file is not yet increased and another +1 because the scale in the file name begins in 1 instead on 0
                        cout<<m[fem][male]<<"\t"<<M;
                    }
                }

            } // close model EM && !massEM
            else{ /// the model is with replacement so the frequencies are constant and there is no need for updates
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        matings[r][c]=nprime*(p1[r]*p2[c]*m[r][c]/M);

                    }
                }
            }


            // write file with the mating table

            if(format==15){
                for(auto& f:obsfems)
                    f/=n1;

                for(auto& f:obsmales){
                    f/=n2;
                }
            }



            ofilename+=NtoS(nf+1+nameshift)+".txt";
            string comments="";
            writediscretemodel(ofilename,programname,comments,format, obsfems,  obsmales, matings);
            ++nf;
            ofilename=dir+filename;

            /// reset to zero if drift
            if(drift){
                fill(obsfems.begin(),obsfems.end(),0.0);
                fill(obsmales.begin(),obsmales.end(),0.0);
            }

            if(EM){ // clean the mating matrix
                for(auto& mrow:matings)
                    std::fill(mrow.begin(), mrow.end(),0);
            }

            if(!fixseed){
                //seed = std::chrono::high_resolution_clock::now().time_since_epoch().count();
                seed=std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count();
                randgen.seed(seed);
                if(verbose) cout << "\n\nSeed: " <<seed<< endl;
            }

        }while(nf<numfiles); // close while
    } // close if onesmplround
    else{ // !onesmplround. The sampling is over different mating rounds where the frequencies are updated. As before but before we do not sample until the end


/// ***************************** FOLLOWING SECTION IS STILL PRIVATE. IT IS NOT DOCUMENTED IN THE MANUAL **********************************

/// ***************************************************************************************************************************************
/// ******************** if onesmplround then mmax<nprime and the model is without replacement and the sampling is over R mating rounds (See Carvajal-Rodriguez 2019, TPB)
/// ***************************************************************************************************************************************
/// The model is forced to be encounter-mating i.e without replacement EM=true
/// The concept of drift previously used for generating the constant population frequencies does NOT MAKE SENSE here for the initial frequencies

        bool EM= true; // detect if the desired model is Encounter-mating type (without replacement models)
        modeltype = abs(modeltype); // detect if the model is with mass encounters
        bool massEM = (modeltype>9) && EM;
        if(massEM){
            int minNsex = min(n1,n2);
            if(mmax < 0){ // was not defined
                mmax = minNsex;

            }

            if(modeltype != 123321)
                modeltype/=10;
            else
                modeltype =0; // now there is random mating under massencounter
        }


        filename+="MateSim_Mmax_";


        if(nullrow<0) // has not been settled by argument. For these models represent the default row with unitary propensity
            nullrow=k1-1;

        if(nullcol<0) // has not been settled by argument. For these models represent the default column with unitary propensity
            nullcol=k2-1;


    /// Check the number of possible matings depending on the population and the pair formation process

        int maxmatings= min(n1,n2);

        if(nprime > maxmatings){

            string msg = "\nToo few population males or females (" +NtoS(maxmatings)+ ") for too many matings (" +NtoS(nprime)+ ")\n";
            cout<<msg<<endl;
            errormsg(maxmatings,"MateSim.err", msg,true,true);

        }

/// The maximum expected number of matings at each round is defined by mmax
        double factor=posfactor;
        if(modeltype==0) factor =1;
        else
        if(modeltype==3 || modeltype==4) factor=choicefactor;
//        else
//        if(modeltype==8 || modeltype==9) factor=max_element({posfactor*choicefactor, posfactor,choicefactor});

        double defaultmij=mmax/factor;

       //if(modeltype==4) defaultmij=mmax;

       // if the model is non-random but the individual model is desired (mmax=1) under the many rounds samplings

//        if(modeltype!=0 &&modeltype!=5 && mmax==1){
//            if(posfactor>=1){ //error because this will produce a random mating
//                string msg = "\nA on random model individual-encounter model (mmax =" +NtoS(mmax)+ ") should define a lower than one posfactor argument (" +NtoS(posfactor)+ ")\n";
//                cout<<msg<<endl;
//                errormsg(maxmatings,"MateSims.err", msg,true,true);
//            }
//            defaultmij=posfactor;
//        }

        // vector for mutual propensities defining the type of model
        vector<vector<double>>m(k1,vector<double>(k2,defaultmij));
        double a=1;

        switch(modeltype){

           case 0: // Neutral M0 models
               for(auto& mr:m)
                for(auto& mij: mr)
                    mij=mmax; // this is the number of matings per round
               filename+=NtoS(mmax)+"_";
               break;

           case 1: // Female sexual selection models with uniform or different frequencies
                // all but the null row could have non-neutral propensity
                a =mmax;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r!=nullrow)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_Factor";
                filename+=NtoS(factor)+"_";

               break;

            // Male sexual selection models with uniform or different frequencies

           case 2: // Male sexual selection models with uniform or different frequencies
                // all but the null col could have non-neutral propensity
                a=mmax;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(c!=nullcol)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_Factor";
                filename+=NtoS(factor)+"_";
               break;

           case 3: // Positive assortative strict (uniform freqs) or mixed
                if(choicefactor<1) errormsg(choicefactor,"MateSim.err", "Wrong < 1 choice-factor for model 3",true,true);
                a= mmax;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r==c)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_Factor";
                filename+=NtoS(factor)+"_";
               break;
           case 4: // Negative assortative strict (uniform) or mixed
                if(choicefactor>1) errormsg(choicefactor,"MateSim.err", "Wrong > 1 choice-factor for model 4",true,true);

                a=mmax;
                for(int r=0; r<(k1);++r){
                    for(int c=0; c<k2; ++c){
                        if(r!=c)
                            m[r][c]=a;
                    }
                }
                filename+=NtoS(a)+"_Factor";
                filename+=NtoS(factor)+"_";
               break;
           case 5: // User defined of preferences
                {
                    string path=""; // error: jump to case label is generated in the next case if this variable is declared out of the block
                    string name="MateSimUserModel.txt";
                    // this could reset k1 and k2
                    ReadUserModels(path,name,  m, k1, k2);

                    // recompute mmax because is user-defined with the model
                    mmax=0;
                    double tempmax=0;
                    for(const auto& mr:m){
                        tempmax=*std::max_element(mr.begin(), mr.end());
                        mmax=std::max(tempmax,mmax);
                    }

                    a=mmax;
                    filename+="User_";
                    break;
                }

            default:
                string msg = "Wrong model number "+NtoS(modeltype) +"\n";
                cout<<msg<<endl;
                errormsg(uniformfreqs,"MateSim.err", msg,true,true);
        } // close switch


        Rmax= nprime;
        //R= min(Rmax, r if sum(matings)>nprime)
/// Predefined model type is changed to adjust to the maximum number of matings per round
        if(mmax>=2){
            if(verbose) cout<<"Mass encounter model with sampling rounds"<<endl;
            massEM=true;

        }
        else{
            massEM=false;
            if(verbose) cout<<"Individual encounter model with sampling rounds"<<endl;
        }

        if(massEM)
            filename+="massEM_";
        else
            filename+="iEM_";

        filename+="RM"+NtoS(modeltype)+"_";
        if(massEM) dirname+="_massEM";
        else dirname+="_iEM";

        if(modeltype!=5)
            dirname+= "_"+ NtoS(modeltype)+"_Mmax"+NtoS(mmax)+"_factor"+NtoS(factor);
        else
            dirname+= "_"+ NtoS(modeltype)+"_User";

/// Set the initial frequencies
/// Here uniform frequencies refers to the initial population frequencies that would change over the rounds

        if(uniformfreqs){
            dirname+="_U_N"+NtoS(n)+"_Osr"+NtoS(osr)+"sample"+NtoS(nprime);
            filename+="U_";
        }
        else if(!userdef){
             dirname+="_nonU_N"+NtoS(n)+"_Osr"+NtoS(osr)+"sample"+NtoS(nprime);
             filename+="nonU_";
            }
            else{
                dirname+="_User_N"+NtoS(n)+"_Osr"+NtoS(osr)+ "sample"+NtoS(nprime);
                filename+="User"+NtoS(n)+"_";
            }


        if(!massEM){

            if(empiricalJRpsi)
                filename+="empRpsi_";
        }
        portablemkdir(dirname);
        dir=dirname+"/";
        string ofilename=dir+filename;

/// Initial population frequencies and initial observed numbers of individuals of each type
        vector<double> p1(k1,0), p2(k2,0);

        vector<double> userp1, userp2;
        vector<double> obsfems(k1,0), obsmales(k2,0);

        if(userdef){
            userp1.resize(k1,0);
            userp2.resize(k2,0);
            string path="";
            string name="MateSimUserFreqs.txt";
            ReadUserFreqs(path,name, k1, k2, userp1, userp2,MPF);
            int acum=0;
            p1=userp1;
            p2=userp2;
            obsfems=p1;
            obsmales=p2;
            //
            for(int f=0; f< (k1-1); ++f){
                obsfems[f]*= n1;
                acum+=obsfems[f];
            }
            obsfems[k1-1] = n1-acum;
            acum=0;
            for(int m=0; m< (k2-1); ++m){
                obsmales[m]*=n2;
                acum+=obsmales[m];
            }
            obsmales[k2-1] = n2-acum;

        }
        else
        if(uniformfreqs){ // if uniform they are not user-def

            double acum=0;
            // generate female population freqs
            for(int f=0; f< (k1-1); ++f ){
                obsfems[f]=(double)n1/(double)k1; // the distribution of the k1 female types is uniform over the n1 females
                acum+=obsfems[f];
            }
            obsfems[k1-1] = n1-acum;

            // population frequencies
            p1=obsfems;
            for(auto& f:p1)
                f/=n1;

            acum=0;
            // generate male population freqs
            for(int f=0; f< (k2-1); ++f ){
                obsmales[f]=(double)n2/(double)k2;
                acum+=obsmales[f];
            }

            obsmales[k2-1] = n2-acum;

            // population frequencies
            p2=obsmales;

            for(auto& f:p2){
                f/=n2;
            }

        } // close if uniformfreqs
        else // non-userdef non-uniform freqs
        { // the frequencies are not uniform and has not been given by the user
                int acum=0;
                // females
                if(k1>1){ /// Dec 2019
                    for(int f=0; f< (k1-1); ++f ){
                        if(acum==n1-1) break;
                        double u=uniform(randgen);
                        while(u<MPF || u> (1.0-MPF)){
                            u=uniform(randgen);
                        }
    //                    int minn1 =max((int)(MPF*n1),2);
    //                    obsfems[f]=max((int)(u*n1),minn1); // at least 2 or n1*maf female of each type
                        obsfems[f]=u*n1;
                        acum+=obsfems[f];
                        if(acum>=n1){obsfems[f]-=(1+acum-n1); acum=n1-1;}
                    }
                    obsfems[k1-1] = n1-acum; // forces at least 1 female
                    /// Dec 2019 shuffle the frequencies because if not when k>2 the obs[0] is in average greater than the others
                    shuffle(std::begin(obsfems), std::end(obsfems), randgen);

                }
                else
                    obsfems[0] = n1;

                p1=obsfems;

                for(auto& f:p1)
                    f/=n1;

                acum=0;
                // males
                if(k2>1){ /// Dec 2019
                    for(int f=0; f< (k2-1); ++f ){
                        if(acum==n2-1) break;
                        double u=uniform(randgen);
                        while(u<MPF || u> (1.0-MPF)){
                            u=uniform(randgen);
                        }
                        //int minn2 =max((int)(MPF*n2),2);
                        //obsmales[f]=max((int)(n2*u),minn2);
                        obsmales[f]=n2*u;
                        acum+=obsmales[f];

                        if(acum>=n2){obsmales[f]-=(1+acum-n2); acum=n2-1;} // at least one male in the last class
                    }
                    obsmales[k2-1]=n2-acum;
                    /// Dec 2019 shuffle the frequencies because if not when k>2 the obs[0] is in average greater than the others
                    shuffle(std::begin(obsmales), std::end(obsmales), randgen);
                }
                else
                    obsmales[0]=n2;

                p2=obsmales;
                for(auto& f:p2){
                    f/=n2;
                }

        } // close userdef && nonuniformfreqs


    /// compute initial mean M0
        double M0=0;
        for(int r=0; r<(k1);++r){
            for(int c=0; c<k2; ++c){
                M0+=(p1[r]*p2[c]*m[r][c]);

            }
        }

/// ******** REPEAT for numfiles: The initial data (obsfem and obsmales) is the same for all the desired files.
        int nf=0;

        double cSigRpti=0, cSigRpsi=0, cSigRpss=0,cSigpti=0, cSigpsi=0, cSigpss=0,cSigRps1=0,cSigRps2=0;
        double aveR=0,avemmax=0,Avemax=0;

       /// Oct 18: vector for accumulating the indexes accumJRPTI compute the JRPTI considering R=r at each round. Each accumulated value is averaged over the runs
        vector<vector<double>> accumJRpti(Rmax,vector<double>(numfiles,missing_value)), accumJRpsi(Rmax,vector<double>(numfiles,missing_value)), accumJRpss(Rmax,vector<double>(numfiles,missing_value));

        vector<double> accumJpti(numfiles,missing_value),accumJpsi(numfiles,missing_value),accumJpss(numfiles,missing_value);

        vector<int> numfilesR(Rmax,0); // count the number of times a given mating round happens for a file. For the individual model is always the number fo files for each round but this is not true for the mass model
        vector<vector<double>> accQr(k1,vector<double>(k2,0));

        do{ // for each file

/// Models

/*
Without replacement: the encounter-mating model (Gimelfarb 1988)
 *
 *     Within this scenario we use the urn scheme (Gimelfarb 1988) for the process of pair formation:
 *     Individual or Mass encounters
 *      Given first mating round r=0; n1 females and n2 males, xi(0) females of type 1 and yj(0) males of type j
 *      and let n = min(n1,n2) the maximum number of pairs that can be arranged in the population.
        The maximum number of matings in each round is mmax. The process will be repeated until nprime matings are obtained
 *      i) numm=0; Compute PIij = qrij*mij and the mean PImean
        ii) make a roulette for selecting Rmax (nprime) females and males
 *      iii) For each of the pairs formed in i) get a uniform number U(0,1).
 *      iii)
 *          Repeat
            if U<= PIij/PImean {++obs[r][i][j], ++mr, ++numm;
            until mmax successful pairs are formed. Then compute mr (<=mmax) stores the number of matings that occurred in this round. And mfri = sum (Obs[r][i][] and mmrj = sum (Obs[r][][j]
            then update: if(numm<nprime){n1 = n1-mr; --n2; p1ri(m) = (xi(r)-mri) /(n1); p2rj(m) = (yj(r)-mmrj) /(n2-mr)} else break}
 *
 *      iv) loop to ii)
*/
/// Generate matings for sample size nprime under different models: EM, massEM...
            bool append=true;

            if(!massEM){ // individual EM model: model 1 from Gimelfarb 1988
if(verbose) cout<<"Beginning individual encounters with sampling rounds"<<endl;


                vector<double> tempobsfems=obsfems, tempobsmales=obsmales;
                vector<vector<vector<double>> > matings(Rmax, vector<vector<double>>(k1,vector<double>(k2,0)) );
                //
                vector<vector<double>>females(Rmax, vector<double>(k1,0));
                vector<vector<double>>males(Rmax,vector<double>(k2,0));

             // because there is  no replacement the frequencies are updated after encounters
                vector<vector<double>>p1_t(Rmax,p1), p2_t(Rmax,p2); // in the case of the individual model the matings are exactly Rmax=nprime

                int nfem=n1, nmale=n2;
                int cm=0; // count matings
                int round=0; // count mating rounds

                vector<double>Mr(Rmax,M0);
                vector<vector<double>> qRprime(k1,vector<double>(k2,0));
                vector<vector<double>>p1Rprime(Rmax,vector<double>(k1,0)), p2Rprime(Rmax,vector<double>(k2,0));

                for(int i=0; i< k1;++i){
                    for(int j=0; j< k2;++j){ // round=0
                        p1Rprime[round][i] +=  p1_t[round][i]* p2_t[round][j]*m[i][j];
                        p2Rprime[round][j] +=  p1_t[round][i]* p2_t[round][j]*m[i][j];
                        // Oct 18:
                        accQr[i][j] = p1_t[round][i]* p2_t[round][j];
                    }
                }

                transform(p1Rprime[round].begin(),p1Rprime[round].end(),p1Rprime[round].begin(), [=](double element){return(element/Mr[round]);} );

                // adjust the frequency to 1
                double acp1=0;
                p1Rprime[round][k1-1] = 1.0 - accumulate(p1Rprime[round].begin(), p1Rprime[round].begin()+k1-1, acp1 );

                transform(p2Rprime[round].begin(),p2Rprime[round].end(),p2Rprime[round].begin(), [=](double element){return(element/Mr[round]);} );
                double acp2=0;
                p2Rprime[round][k2-1] = 1.0 - accumulate(p2Rprime[round].begin(), p2Rprime[round].begin()+k2-1, acp2 );

                /// real PRprime following the model in eq (3)
                vector<vector<double>> PRprime(k1,p2Rprime[round]);

                // initiallize PRprime
                for(int i=0; i< k1;++i)
                    for(int j=0; j< k2;++j)
                        PRprime[i][j]*=p1Rprime[round][i];

                /// estimated PRrime by countings (this is the one used in the mass model)
                vector<vector<double>> ePRprime(k1,vector<double>(k2,0));
                /// Sexual selection by sex
                vector<double>avep1R(p1_t[0]),avep2R(p2_t[0]),avep1PrimeR(p1Rprime[0]),avep2PrimeR(p2Rprime[0]);

/// Accumulate values through rounds to see the evolution of the indices. So we set R=1 for the first round R=2 for the second and R=Rmax at the end.
                //vector<vector<double>> accPRprime(k1,p2Rprime[round]);

// pairs are formed step by step
                do{ // repeat until nprime matings are reached

//cout<<nfem<<"\t"<<nmale<<endl;                    // get 1 female
                    int fem = roulette(p1_t[round],append, females[round]); // in females the females selected for mating are stored

                    int male = roulette(p2_t[round],append, males[round]);    // same in males

                    if(uniform(randgen) <= m[fem][male]/Mr[round] ){ // this pair successfully mate
                        ++cm;
                        ++matings[round][fem][male];
                        ++qRprime[fem][male];
                        ++ePRprime[fem][male];
                        //Oct 18:
                        double tempQrprime = 0; // Qprime considering R=accumulated rounds
                        double tempPrprime = 0; //Pprime considering  R=accumulated rounds

                        /// Oct 18: This store the value for each R=round and accumulate over replicates
//cout<<tqrp<<"\t"<<tprp<<"\t"<<tqr<<endl;
                        for(int fem=0; fem<k1; ++fem){
                            for(int male=0; male<k2;++male){
                                tempQrprime = max(qRprime[fem][male],0.5);
                                tempPrprime = max(PRprime[fem][male],0.5);
/*
if(cm==nprime){

    cout<<tempQrprime<<"\t"<<tempPrprime<<"\t"<<accQr[fem][male]<<endl;
}
*/
                                if(accumJRpti[cm-1][nf]==missing_value){ // first time
                                    accumJRpti[cm-1][nf] = (tempQrprime/cm - accQr[fem][male]/cm)* log(tempQrprime / accQr[fem][male]);
                                }
                                else{
                                    accumJRpti[cm-1][nf] += (tempQrprime/cm - accQr[fem][male]/cm)* log(tempQrprime / accQr[fem][male]);
                                }
                                if(accumJRpsi[cm-1][nf]==missing_value){
                                    accumJRpsi[cm-1][nf] = (tempQrprime/cm -tempPrprime/cm)*log(tempQrprime/tempPrprime);
                                }
                                else{
                                    accumJRpsi[cm-1][nf] += (tempQrprime/cm -tempPrprime/cm)*log(tempQrprime/tempPrprime);
                                }
                                if(accumJRpss[cm-1][nf]==missing_value){
                                    accumJRpss[cm-1][nf] = (tempPrprime/cm - accQr[fem][male]/cm)* log(tempPrprime / accQr[fem][male]);
                                }
                                else{
                                    accumJRpss[cm-1][nf] += (tempPrprime/cm - accQr[fem][male]/cm)* log(tempPrprime / accQr[fem][male]);
                                }

                            } // close for male
                        } // close for female

                        ++numfilesR[cm-1]; // this  count the number of files that reach a given mating round

// under the individual model if computed from observations p1Rprime, p2Rprime are 1 or 0 and so PRprime matches exactly
//the number of matings over the total matings (because the number of rounds matches the total matings and in fact we cannot compute the marginals
// at each round) and so PRprime is equal to QRprime
//and JRpsi is by definition 0 so under the individual model p1Rprime and p2Rprime must be estimated from the a priori formulae (eq 3 in the article)
// so in this case we use the a prioristic expression

//if(verbose) cout<<"Mating "<<cm<<endl;

                        if(cm<nprime){
                        // update values

                            --nfem; --nmale;
                            tempobsfems[fem] -= females[round][fem];
                            tempobsmales[male] -= males[round][male];
                            //if(r<Rmax){ // r has been updated and the last population frequencies are not need to be computed
                          /// population frequencies at the next round (update first round counter)
                            ++round;

                            double p1acum=0;
                            for(int i=0; i< (k1-1); ++i){
                                p1_t[round][i] = (tempobsfems[i]) /(nfem);
                                p1acum+=p1_t[round][i];
                            }
                            p1_t[round][k1-1] = 1.0 -p1acum;

                            double p2acum=0;
                            for(int j=0; j< (k2-1); ++j){
                                p2_t[round][j] = (tempobsmales[j]) /(nmale);
                                p2acum+=p2_t[round][j];
                            }
                            p2_t[round][k2-1] = 1.0-p2acum;

                            //}

                            // update the mean preference (PI(k) in formula 10 in Gimelfarb 88)
                            double M=0;
                            for(int r=0; r<(k1);++r){
                                for(int c=0; c<k2; ++c){
                                    // Oct 18: accumulate the phenotype frequencies by rounds to paint the indices against rounds
                                    accQr[r][c] += p1_t[round][r]* p2_t[round][c];

                                    M+=(p1_t[round][r]*p2_t[round][c]*m[r][c]);

                                }
                            }
                            Mr[round]=M;

                            for(int i=0; i< k1;++i){
                                for(int j=0; j< k2;++j){
                                    p1Rprime[round][i] +=  p1_t[round][i]* p2_t[round][j]*m[i][j];
                                    p2Rprime[round][j] +=  p1_t[round][i]* p2_t[round][j]*m[i][j];
                                }
                            }

                            transform(p1Rprime[round].begin(),p1Rprime[round].end(),p1Rprime[round].begin(), [=](double element){return(element/Mr[round]);} );
                            transform(p2Rprime[round].begin(),p2Rprime[round].end(),p2Rprime[round].begin(), [=](double element){return(element/Mr[round]);} );
                // adjust the frequency to 1
                            acp1=0;
                            p1Rprime[round][k1-1] = 1.0 - accumulate(p1Rprime[round].begin(), p1Rprime[round].begin()+k1-1, acp1 );
                            acp2=0;
                            p2Rprime[round][k2-1] = 1.0 - accumulate(p2Rprime[round].begin(), p2Rprime[round].begin()+k2-1, acp2 );

                            for(int i=0; i< k1;++i){
                                for(int j=0; j< k2;++j){
                                    PRprime[i][j]+= (p1Rprime[round][i]*p2Rprime[round][j]);
                                }
                                // compute averages for JRS1
                                avep1R[i]+=p1_t[round][i];
                                avep1PrimeR[i] +=p1Rprime[round][i];
                            }
                            // compute averages for JRS2
                            for(int j=0; j< k2;++j){
                                avep2R[j]+=p2_t[round][j];
                                avep2PrimeR[j] +=p2Rprime[round][j];
                            }

//if(verbose) cout<<"Round "<<round<<endl;

                        }
                        else{ // the nprime matings were already obtained
                            break;
                        }
                    } // close if successful mating
                    else{ // the mate did not happen and the fem and male return to the urn

                        --females[round][fem];
                        --males[round][male];

//if(verbose){
//  for(const auto& pi:p1_t[round])
//    cout<<"p1i "<<pi<<"\t";
//  for(const auto& pj:p2_t[round])
//    cout<<"p2j"<<pj<<"\t";
//
//  cout<<"\nmij "<<m[fem][male] <<" Mr "<<Mr[round]<<" mij/Mr "<<m[fem][male]/Mr[round]<<endl;
//
//}
                    }

                }while(cm<nprime);

                avemmax+=1;
                Avemax=avemmax;
                aveR+=Rmax;
if(verbose) cout<<NtoS(cm)<<" individual mating rounds performed"<<endl;
/// Compute information indexes

                // Make relative frequencies
//                transform(p1Rprime.begin(),p1Rprime.end(),p1Rprime.begin(), [=](double element){return(element/nprime);} );
//                transform(p2Rprime.begin(),p2Rprime.end(),p2Rprime.begin(), [=](double element){return(element/nprime);} );


                for(auto & P: PRprime){
                    transform(P.begin(),P.end(),P.begin(), [=](double element){return(element/nprime);} );
                }

                for(auto & q: qRprime){ // if there are empty observations replace by 0.5 (as in Carvajal-Rodriguez & Rolan-Alvarez 2006)
                    transform(q.begin(),q.end(),q.begin(), [=](double element){if(element==0) element=0.5; return(element/nprime);} );
                }

                for(auto & P: ePRprime){
                    transform(P.begin(),P.end(),P.begin(), [=](double element){return(element/nprime);} );
                }


                transform(avep1R.begin(),avep1R.end(),avep1R.begin(), [=](double element){if(element==0) element=0.5; return(element/nprime);} );
                transform(avep1PrimeR.begin(),avep1PrimeR.end(),avep1PrimeR.begin(), [=](double element){if(element==0) element=0.5; return(element/nprime);} );

                transform(avep2R.begin(),avep2R.end(),avep2R.begin(), [=](double element){if(element==0) element=0.5; return(element/nprime);} );
                transform(avep2PrimeR.begin(),avep2PrimeR.end(),avep2PrimeR.begin(), [=](double element){if(element==0) element=0.5; return(element/nprime);} );

                vector<vector<double>> qR(k1,vector<double>(k2,0));
                for(int r=0; r<nprime; ++r){
                    for(int fem=0; fem<k1; ++fem){
                        for(int male=0; male<k2;++male){
                            qR[fem][male]+= p1_t[r][fem]*p2_t[r][male];
                        }
                    }
                }

                // Make relative frequencies
                double accumq=0;
                for(int fem=0; fem<k1; ++fem)
                    for(int male=0; male<k2;++male){
                        qR[fem][male]/= nprime;
                        if((fem!=(k1-1)) || (male!=(k2-1)) )
                            accumq+=qR[fem][male];
                    }

                // correct precision error for summing up to 1
                qR[k1-1][k2-1] = 1.0 -accumq;

                vector<double>p1prime(k1,0),p2prime(k2,0);
                for(int fem=0; fem<k1; ++fem){
                    for(int male=0; male<k2;++male){
                        p1prime[fem] += qRprime[fem][male];
                        p2prime[male] += qRprime[fem][male];
                    }
                }
                vector<vector<double>>q(k1,p2), Pprime(k1,p2prime);
                double JRpti=0, Jpti=0, JRpsi=0, Jpsi=0, JRpss=0, Jpss=0;

                /// use the empirical estimate of Prime or use the theoretical
                if(empiricalJRpsi){

                    PRprime=ePRprime;
                }

                for(int fem=0; fem<k1; ++fem){
                    for(int male=0; male<k2;++male){
                        JRpti+= (qRprime[fem][male] - qR[fem][male])*log(qRprime[fem][male]/qR[fem][male]);
                        q[fem][male]*=p1[fem];
                        Jpti+= (qRprime[fem][male] - q[fem][male])*log(qRprime[fem][male]/q[fem][male]);

                        JRpsi+= (qRprime[fem][male] - PRprime[fem][male])*log(qRprime[fem][male]/PRprime[fem][male]);
                        Pprime[fem][male]*=p1prime[fem];
                        Jpsi+= (qRprime[fem][male] - Pprime[fem][male])*log(qRprime[fem][male]/Pprime[fem][male]);

                        JRpss+= (PRprime[fem][male] - qR[fem][male])*log(PRprime[fem][male]/qR[fem][male]);

                        Jpss+= (Pprime[fem][male] - q[fem][male])*log(Pprime[fem][male]/q[fem][male]);


                    } // close for male
                } // close for fem

                int K = k1*k2;

                double pJRpti =getChiPval(nprime*JRpti,K-1);
                double pJpti = getChiPval(nprime*Jpti,K-1);

                double pJRpsi = 1;
                double pJpsi = 1;

                double pJRpss = 1;
                double pJpss = 1;

                double JRs1=0,JRs2=0,pJRs1=1,pJRs2=1;

                for(int i=0;i<k1;++i){
                    JRs1+= (avep1PrimeR[i]-avep1R[i])*log(avep1PrimeR[i] / avep1R[i]);
                }

                for(int j=0;j<k2;++j){
                    JRs2+= (avep2PrimeR[j]-avep2R[j])*log(avep2PrimeR[j] / avep2R[j]);
                }

                if(pJRpti<=SL){
                    ++cSigRpti;
                    pJRpsi = getChiPval(nprime*JRpsi,K-k1-k2+1);
                    if(pJRpsi<=SL){
                        ++cSigRpsi;
                    }
                    pJRpss =getChiPval(nprime*JRpss,k1+k2-2);
                    if(pJRpss<=SL){
                        ++cSigRpss;
                        pJRs1 =getChiPval(nprime*JRs1,k1-1);
                        if(pJRs1<=SL)
                            ++cSigRps1;

                        pJRs2 =getChiPval(nprime*JRs2,k2-1);
                        if(pJRs2<=SL)
                            ++cSigRps2;
                    }
                 }

                if(pJpti<=SL){
                    ++cSigpti;
                    pJpsi =getChiPval(nprime*Jpsi,K-k1-k2+1);
                    if(pJpsi<=SL){
                        ++cSigpsi;
                    }
                    pJpss =getChiPval(nprime*Jpss,k1+k2-2);
                    if(pJpss<=SL){
                        ++cSigpss;

                    }
                 }

    /// Oct 18: Store the values of Jpti etc for each run
                accumJpti[nf] = Jpti;
                accumJpsi[nf] = Jpsi;
                accumJpss[nf] = Jpss;


if(verbose){
cout<<nf<<endl;

//cout<<JRpti<<" = "<<JRpss<<" + "<<JRpsi<<" = "<< JRpss+JRpsi <<endl;

//cout<<JRpss<<" = "<<JRs1<<" + "<<JRs2<<" = "<< JRs1+JRs2 <<endl;
double ER0 = JRpti - (JRpss+JRpsi);
cout<<"E0R "<<ER0<<endl;

}
//assert(fabs(JRpss - (JRs1+JRs2)) < 1e-2 );

            // write file with the mating table by rounds

                ofilename+=NtoS(nf+1+nameshift)+".txt";
                string comments="";
                format= 16;

                vector<double>JR{JRpti,JRpsi,JRpss,JRs1,JRs2};
                vector<double>JRpvals{pJRpti,pJRpsi,pJRpss,pJRs1,pJRs2};
                vector<double>J{Jpti,Jpsi,Jpss};
                vector<double>Jpvals{pJpti,pJpsi,pJpss};


if(verbose && !onlyresume) cout<<"Writing output"<<endl;
                if(!onlyresume)
                    writeRoundsTable(ofilename,programname,comments,format, p1_t,p2_t,matings,qRprime,qR,JR,JRpvals,J,Jpvals);

            } // close !massEM
            else if(massEM){
if(verbose) cout<<"Beginning Mass encounters with sampling rounds"<<endl;

                // pairs are formed in mass over R rounds to a maximum of Rmax rounds
                //vector<double> tempobsfems=obsfems, tempobsmales=obsmales;
                vector<vector<vector<double>> > matings;
                matings.reserve(Rmax);
                matings.push_back(vector<vector<double>>(k1,vector<double>(k2,0)));
                //

             // because there is  no replacement the frequencies are updated after encounters
                vector<vector<double>>p1_t, p2_t; //
                p1_t.reserve(Rmax);
                p1_t.push_back(p1);
                p2_t.reserve(Rmax);
                p2_t.push_back(p2);

                int nfem=n1, nmale=n2;

                int cm=0; // count matings
                int round=0; // count mating rounds

                for(int i=0; i< k1;++i){
                    for(int j=0; j< k2;++j){ // round=0
                                         // Oct 18:
                        accQr[i][j] = p1_t[round][i]* p2_t[round][j];
                    }
                }


                vector<vector<double>> qRprime(k1,vector<double>(k2,0));

                vector<vector<double>>p1rprime, p2rprime; // for store over runs (this was not necessary under individual encounters because each p1Rprime_i was always 0 or 1)

                p1rprime.reserve(Rmax);
                p2rprime.reserve(Rmax);
                p1rprime.push_back(vector<double>(k1,0));
                p2rprime.push_back(vector<double>(k2,0));

                vector<vector<double>> PRprime(k1,vector<double>(k2,0));

                vector<double>Mr;
                Mr.reserve(Rmax);
                Mr.push_back(M0);

                vector<double>availfems = obsfems;
                vector<double>availmales = obsmales;


                do{

                    int mmin=min(nfem,nmale);
                    int matR=min((int)mmax,mmin);
                    vector<int>femindexes((int)matR,-1);
                    vector<int>maleindexes((int)matR,-1);
                    // get mmax females and males if there are if not get the min(nfem,nmale)
                    roulette(matR,p1_t[round], availfems, femindexes );
                    roulette(matR,p2_t[round],availmales, maleindexes );

                    // perform the matings
                    int contmr=0; // count mating by round
                    for(int mt=0; mt<matR; ++mt){

                        int fem = femindexes[mt];
                        int male = maleindexes[mt];

                        if(uniform(randgen) <= m[fem][male]/Mr[round] ){ // this pair successfully mate

                            ++cm;
                            ++matings[round][fem][male];
                            ++qRprime[fem][male];
                            ++contmr;

                        // here in the mass encounter the frequencies prime should be relativized by contmr
                            ++p1rprime[round][fem];
                            ++p2rprime[round][male];


                            if(cm<nprime){
                            // update number of available females and males
                                --nfem; --nmale;

                            }
                            else{ // the nprime matings were already obtained
                                break;
                            }

                        }
                        else{ // the mate did not happen and the fem and male return to the urn
                            ++availfems[fem];
                            ++availmales[male];
                        }

                    } // close for mt

                    /// After the mass encounter divide the counting by the number of matings

                    if(contmr){

                        ++numfilesR[round];

                        avemmax+=contmr;
                        transform(p1rprime[round].begin(),p1rprime[round].end(),p1rprime[round].begin(), [=](double element){return(element/contmr);} );
                        transform(p2rprime[round].begin(),p2rprime[round].end(),p2rprime[round].begin(), [=](double element){return(element/contmr);} );

                        for(int fem=0;fem <k1;++fem){
                            for(int male=0; male<k2; ++male){
                                PRprime[fem][male] += p1rprime[round][fem]*p2rprime[round][male];
                            }
                        }

                        /// Oct 18: This store the value for each R=round and accumulate over replicatse

                        double tempQrprime = 0; // Qprime considering R=accumulated rounds
                        double tempPrprime = 0; //Pprime considering  R=accumulated rounds

                        /// Oct 18: This store the value for each R=round and accumulate over replicatse
//cout<<tqrp<<"\t"<<tprp<<"\t"<<tqr<<endl;
                        for(int fem=0; fem<k1; ++fem){
                            for(int male=0; male<k2;++male){

                                tempQrprime = max(qRprime[fem][male],0.5)/cm;
                                tempPrprime = max(PRprime[fem][male],0.5)/ (round+1);

                                if(accumJRpti[round][nf]== missing_value){ // first time

                                    accumJRpti[round][nf] = (tempQrprime - accQr[fem][male]/(round+1))* log(tempQrprime / (accQr[fem][male]/(round+1)));

                                     // count the number of files in that this mating round occurs
                                }
                                else{ // accumulate
                                    accumJRpti[round][nf] += (tempQrprime - accQr[fem][male]/(round+1))* log(tempQrprime / (accQr[fem][male]/(round+1)));

                                }

                                if(accumJRpsi[round][nf] == missing_value){
                                    accumJRpsi[round][nf] = (tempQrprime -tempPrprime)*log(tempQrprime/tempPrprime);
                                }
                                else{
                                    accumJRpsi[round][nf] += (tempQrprime -tempPrprime)*log(tempQrprime/tempPrprime);
                                }

                                if(accumJRpss[round][nf]== missing_value){
                                    accumJRpss[round][nf] = (tempPrprime - accQr[fem][male]/(round+1))* log(tempPrprime / (accQr[fem][male]/(round+1)));
                                }
                                else{
                                    accumJRpss[round][nf] += (tempPrprime - accQr[fem][male]/(round+1))* log(tempPrprime / (accQr[fem][male]/(round+1)));
                                }
                            } // close for male
                        } // close for female



                    /// After the mass encounter update for the next round provided that some mating occurred (contmr>0)
                        if(cm<nprime){

//cout<<"Round "<<round<<endl;

                            // accumulate the number of mating females and males
//                            for(int mt=0; mt<matR; ++mt){
//
//                                int fem = femindexes[mt];
//                                int male = maleindexes[mt];
//
////                                tempobsfems[fem] -= females[round][fem];
////                                tempobsmales[male] -= males[round][male];
////cout<<availmales[male]<<"\t"<<tempobsmales[male]<<"\t"<<males[round][male]<<"\t"<<nmale<<endl;
//                            } // close for mt

                            ++round;
//cout<<round<<" cm= "<< cm<<endl;
                            matings.push_back(vector<vector<double>>(k1,vector<double>(k2,0)));
                            p1_t.push_back(vector<double>(k1,0));
                            p2_t.push_back(vector<double>(k2,0));
                            p1rprime.push_back(vector<double>(k1,0));
                            p2rprime.push_back(vector<double>(k2,0));

                            // update frequencies

                            double p1acum=0;
                            for(int i=0; i< (k1-1); ++i){
                                p1_t[round][i] = (availfems[i]) /(nfem);
                                p1acum+=p1_t[round][i];
                            }
                            p1_t[round][k1-1] = 1.0 -p1acum;
assert(nfem==nmale);
//cout<<endl;
//for(int i=0; i< (k1-1); ++i)
//    cout<<"\t"<<matings[round-1][i][0]<<"+"<<matings[round-1][i][1]<<"\t"<<availfems[i]<<"\t"<<nfem<<"\t"<<p1_t[round][i]<<"\t";
//cout<<endl;
//cout<<endl;
                            double p2acum=0;
                            for(int j=0; j< (k2-1); ++j){
                                p2_t[round][j] = (availmales[j]) /(nmale);
                                p2acum+=p2_t[round][j];
                            }
                            p2_t[round][k2-1] = 1.0-p2acum;

//for(auto&p2:p2_t[round])
//    cout<<p2<<"\t";
//cout<<endl;

                        // update the mean M
                            double M=0;
                            for(int r=0; r<(k1);++r){
                                for(int c=0; c<k2; ++c){
                                    M+=(p1_t[round][r]*p2_t[round][c]*m[r][c]);

                                // Oct 18: accumulate the phenotype frequencies by rounds to paint the indices against rounds
                                    accQr[r][c] += p1_t[round][r]* p2_t[round][c];

                                }
                            }
//cout<<M<<endl;
                            Mr.push_back(M);

                        } // close if cm < nprime

                    } // close if contmr
//cout<<cm<<endl;

                }while(cm<nprime);


//exit(2);
/// Compute information indexes
                int rounds = Mr.size();
                Avemax+= avemmax/rounds;
                avemmax=0;
                aveR+=rounds;

if(verbose) cout<<rounds<<" mating rounds performed"<<endl;
//cout<<round<<endl;
assert(rounds==round+1); // because first is round=0
assert(p1rprime.size()==(unsigned)rounds);
assert(p2rprime.size()==(unsigned)rounds);

                // Compute averages for JRS1 and JRS2 and make relative frequencies
                vector<double>avep1R(k1,0), avep2R(k2,0);
                vector<double>p1Rprime(k1,0);
                vector<double>p2Rprime(k2,0);


                for(int r=0; r<rounds;++r){

                    for(int fem=0; fem<k1;++fem){
                        avep1R[fem]+=p1_t[r][fem];
                        p1Rprime[fem]+= p1rprime[r][fem];
                    }

                    for(int male=0; male<k2; ++male){
                        avep2R[male]+= p2_t[r][male];
                        p2Rprime[male]+= p2rprime[r][male];
                    }

                }
                // if there are empty observations replace by 0.5 (as in Carvajal-Rodriguez & Rolan-Alvarez 2006) and get the relative frequency

                transform(p1Rprime.begin(),p1Rprime.end(),p1Rprime.begin(), [=](double element){if(element==0) element=0.5;return(element/rounds);} );
                transform(p2Rprime.begin(),p2Rprime.end(),p2Rprime.begin(), [=](double element){if(element==0) element=0.5;return(element/rounds);} );

                transform(avep1R.begin(),avep1R.end(),avep1R.begin(), [=](double element){if(element==0) element=0.5;return(element/rounds);} );
                transform(avep2R.begin(),avep2R.end(),avep2R.begin(), [=](double element){if(element==0) element=0.5;return(element/rounds);} );

                for(auto & P: PRprime){
                    transform(P.begin(),P.end(),P.begin(), [=](double element){if(element==0) element=0.5;return(element/rounds);} );
                }

                for(auto & q: qRprime){ // if there are empty observations replace by 0.5 (as in Carvajal-Rodriguez & Rolan-Alvarez 2006)
                    transform(q.begin(),q.end(),q.begin(), [=](double element){if(element==0) element=0.5; return(element/nprime);} );
                }

                vector<vector<double>> qR(k1,vector<double>(k2,0));
                for(int r=0; r<rounds; ++r){
                    for(int fem=0; fem<k1; ++fem){
                        for(int male=0; male<k2;++male){
//cout<<p1_t[r][fem]<<"\t"<<p2_t[r][male] <<"\t"<< qR[fem][male]<<endl;
                            qR[fem][male]+= p1_t[r][fem]*p2_t[r][male];


                        }
                    }
                }


//                for(int fem=0; fem<k1; ++fem){
//                    for(int male=0; male<k2;++male){
////cout<<p1_t[r][fem]<<"\t"<<p2_t[r][male] <<"\t"<< qR[fem][male]<<endl;
//                        qR[fem][male]= avep1R[fem]*avep2R[male]*R;
//                    }
//                }
//
//                for(int fem=0; fem<k1; ++fem){
//                    for(int male=0; male<k2;++male){
////cout<<p1_t[r][fem]<<"\t"<<p2_t[r][male] <<"\t"<< qR[fem][male]<<endl;
//                        PRprime[fem][male]= p1Rprime[fem]*p2Rprime[male]*R;
//                    }
//                }

                // Make relative frequencies
                double accumq=0;
                for(int fem=0; fem<k1; ++fem)
                    for(int male=0; male<k2;++male){
                        qR[fem][male]/= rounds;
                        if((fem!=(k1-1)) || (male!=(k2-1)) )
                            accumq+=qR[fem][male];
                    }

                // correct precision error for summing up to 1
                qR[k1-1][k2-1] = 1.0 -accumq;


                vector<double>p1prime(k1,0),p2prime(k2,0);
                for(int fem=0; fem<k1; ++fem){
                    for(int male=0; male<k2;++male){
                        p1prime[fem] += qRprime[fem][male];
                        p2prime[male] += qRprime[fem][male];
                    }
                }
                vector<vector<double>>q(k1,p2), Pprime(k1,p2prime);
                double JRpti=0, Jpti=0, JRpsi=0, Jpsi=0, JRpss=0, Jpss=0;
                for(int fem=0; fem<k1; ++fem){
                    for(int male=0; male<k2;++male){
                        JRpti+= (qRprime[fem][male] - qR[fem][male])*log(qRprime[fem][male]/qR[fem][male]);
                        q[fem][male]*=p1[fem];
                        Jpti+= (qRprime[fem][male] - q[fem][male])*log(qRprime[fem][male]/q[fem][male]);

                        JRpsi+= (qRprime[fem][male] - PRprime[fem][male])*log(qRprime[fem][male]/PRprime[fem][male]);
                        Pprime[fem][male]*=p1prime[fem];
                        Jpsi+= (qRprime[fem][male] - Pprime[fem][male])*log(qRprime[fem][male]/Pprime[fem][male]);

                        JRpss+= (PRprime[fem][male] - qR[fem][male])*log(PRprime[fem][male]/qR[fem][male]);

                        Jpss+= (Pprime[fem][male] - q[fem][male])*log(Pprime[fem][male]/q[fem][male]);

                    } // close for male
                } // close for fem


//cout<<JRpti<<"\t"<<JRpsi<<"\t"<<JRpss<<"\t"<<Jpti<<"\t"<<Jpsi<<"\t"<<Jpss<<endl;
//for(auto& Q:qR){
//    for(auto& q:Q){
//        cout<<q<<"\t";
//    }
//    cout<<endl;
//}

                double JRs1=0,JRs2=0;
                /// correction factor to correct the bias to equalize the sum of products to the produc of the sum

                double cfactor=0, cfactorprime=0;

                for(int fem=0; fem<k1; ++fem){
                    for(int male=0; male<k2;++male){
                        cfactor+= avep1R[fem]*(1.0-avep2R[male]);
                        cfactorprime+= p1Rprime[fem]*(1-p2Rprime[male]);
                    }
                }
                for(int i=0;i<k1;++i){
                    JRs1+= (p1Rprime[i]-avep1R[i])*log(p1Rprime[i] / avep1R[i]);
                }

                for(int j=0;j<k2;++j){
                    JRs2+= (p2Rprime[j]-avep2R[j])*log(p2Rprime[j] / avep2R[j]);
                }
if(verbose){
cout<<nf<<endl;

//cout<<JRpti<<" = "<<JRpss<<" + "<<JRpsi<<" = "<< JRpss+JRpsi <<endl;

//cout<<JRpss<<" = "<<JRs1<<" + "<<JRs2<<" = "<< JRs1+JRs2 <<endl;
double ER0 = JRpti - (JRpss+JRpsi);
cout<<"E0R "<<ER0<<endl;

}

                int K = k1*k2;

                double pJRpti =getChiPval(nprime*JRpti,K-1);
                double pJpti = getChiPval(nprime*Jpti,K-1);

                double pJRpsi = 1;
                double pJpsi = 1;

                double pJRpss = 1;
                double pJpss = 1;

                double pJRs1=1,pJRs2=1;
                if(pJRpti<=SL){
                    ++cSigRpti;
                    pJRpsi = getChiPval(nprime*JRpsi,K-k1-k2+1);
                    if(pJRpsi<=SL){
                        ++cSigRpsi;
                    }
                    pJRpss =getChiPval(nprime*JRpss,k1+k2-2);
                    if(pJRpss<=SL){
                        ++cSigRpss;

                        pJRs1 =getChiPval(nprime*JRs1,k1-1);
                        if(pJRs1<=SL)
                            ++cSigRps1;

                        pJRs2 =getChiPval(nprime*JRs2,k2-1);
                        if(pJRs2<=SL)
                            ++cSigRps2;

                    }
                 }

                if(pJpti<=SL){
                    ++cSigpti;
                    pJpsi =getChiPval(nprime*Jpsi,K-k1-k2+1);
                    if(pJpsi<=SL){
                        ++cSigpsi;
                    }
                    pJpss =getChiPval(nprime*Jpss,k1+k2-2);
                    if(pJpss<=SL){
                        ++cSigpss;
                    }
                 }


//assert(fabs(JRpti -(JRpss+JRpsi))< 1e-1);
//assert(fabs(JRpss - (JRs1+JRs2)) < 0.01 );

    /// Oct 18: Store the values of Jpti etc for each run
                accumJpti[nf] = Jpti;
                accumJpsi[nf] = Jpsi;
                accumJpss[nf] = Jpss;

            // write file with the mating table by rounds

                ofilename+=NtoS(nf+1+nameshift)+".txt";
                string comments="";
                format= 16;

                vector<double>JR{JRpti,JRpsi,JRpss,JRs1,JRs2};
                vector<double>JRpvals{pJRpti,pJRpsi,pJRpss,pJRs1,pJRs2};
                vector<double>J{Jpti,Jpsi,Jpss};
                vector<double>Jpvals{pJpti,pJpsi,pJpss};
if(verbose && !onlyresume) cout<<"Writing output"<<endl;
                if(!onlyresume)
                    writeRoundsTable(ofilename,programname,comments,format, p1_t,p2_t,matings,qRprime,qR,JR,JRpvals,J,Jpvals);

            } // close massEM

            ++nf;
            ofilename=dir+filename;

        }while(nf<numfiles); // close while

        // Oct 18:
        double aveJpti=0,sdJpti=0;
        avesd(accumJpti,aveJpti, sdJpti, (double)missing_value);
        double aveJpsi=0,sdJpsi=0;
        avesd(accumJpsi,aveJpsi, sdJpsi, (double)missing_value);
        double aveJpss=0,sdJpss=0;
        avesd(accumJpss,aveJpss, sdJpss, (double)missing_value);

//        for(int r=0; r<Rmax; ++r){
//            if(numfilesR[r]){ // if there are at least one file in which this round occurred
//                accumJRpti[r]/=numfilesR[r];
//                accumJRpsi[r]/=numfilesR[r];
//                accumJRpss[r]/=numfilesR[r];
//            }
//        }

        vector<double> aveJRpti(Rmax,missing_value),sdJRpti(Rmax, missing_value);

        for(int r=0; r<Rmax;++r){
            avevar(accumJRpti[r],aveJRpti[r], sdJRpti[r], (double)missing_value);
        }

        vector<double> aveJRpsi(Rmax,missing_value),sdJRpsi(Rmax, missing_value);

        for(int r=0; r<Rmax;++r){
            avevar(accumJRpsi[r],aveJRpsi[r], sdJRpsi[r], (double)missing_value);
        }

        vector<double> aveJRpss(Rmax,missing_value),sdJRpss(Rmax, missing_value);

        for(int r=0; r<Rmax;++r){
            avesd(accumJRpss[r],aveJRpss[r], sdJRpss[r], (double)missing_value);
        }

        aveR/= numfiles;
        Avemax/= numfiles;
//cout<<Avemax<<endl;
        cSigRpti/=numfiles;
        cSigRpsi/=numfiles;
        cSigRpss/=numfiles;

        cSigpti/=numfiles;
        cSigpsi/=numfiles;
        cSigpss/=numfiles;

        cSigRps1/=numfiles;
        cSigRps2/=numfiles;

        ofilename+=".txt";
        string comments="";
        writeEMSignificances(ofilename,programname,comments,massEM,numfiles,n1,n2,nprime,SL,Avemax,aveR,cSigRpti,cSigRpsi,cSigRpss,cSigRps1,cSigRps2,cSigpti, cSigpsi,cSigpss,aveJRpti,sdJRpti,aveJRpsi,sdJRpsi,aveJRpss,sdJRpss,aveJpti, sdJpti,aveJpsi, sdJpsi, aveJpss, sdJpss, numfilesR);
    } // close else !onesmplround


    } // close if discrete


	return 0;
} // close try
catch(const std::invalid_argument& e ){
    cout<<e.what();
    return 1;
}
catch(const string& msg){
    cout<<msg<<endl; return 1;
}

}



